"""
File       : MSPileupObj_t.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: Unit tests for MicorService/MSPileup/DataStructs/MSPileupObj.py module
"""

# system modules
import time
import unittest

# WMCore modules
from WMCore.MicroService.MSPileup.DataStructs.MSPileupObj import MSPileupObj
from WMCore.MicroService.Tools.Common import getMSLogger


class MSPileupObjTest(unittest.TestCase):
    """Unit test for MSPileupObj module"""

    def setUp(self):
        """
        setup Unit tests
        """
        self.logger = getMSLogger(False)

    def testMSPileupObj(self):
        """
        Unit test for correct MSPileupObj data
        """
        expectedRSEs = ['rse1', 'rse2']
        fullReplicas = 0
        campaigns = ['c1', 'c2']
        data = {
            'pileupName': '/lksjdfkls/lkjaslkdjlas/PREFIX',
            'pileupType': 'classic',
            'expectedRSEs': expectedRSEs,
            'currentRSEs': expectedRSEs,
            'fullReplicas': fullReplicas,
            'campaigns': campaigns,
            'containerFraction': 1.0,
            'replicationGrouping': "ALL",
            'active': True,
            'pileupSize': 0,
            'customName': '',
            'transition': [],
            'ruleIds': []}
        obj = MSPileupObj(data, validRSEs=expectedRSEs)
        for key in ['insertTime', 'lastUpdateTime', 'activatedOn', 'deactivatedOn']:
            self.assertNotEqual(obj.data[key], 0)
        self.assertEqual(obj.data['expectedRSEs'], expectedRSEs)
        self.assertEqual(obj.data['fullReplicas'], fullReplicas)
        self.assertEqual(obj.data['campaigns'], campaigns)

        # check validation functions
        self.assertEqual(obj.validateRSEs(['bla']), False)
        self.assertEqual(obj.validateRSEs(expectedRSEs), True)

        # let's turn off RSEs and check that we'll get exception while
        # creating MSPileupObj
        wrongData = dict(data)
        wrongData['expectedRSEs'] = []
        try:
            MSPileupObj(wrongData, validRSEs=expectedRSEs)
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # let's use non complaint pileupName
        wrongData = dict(data)
        wrongData['pileupName'] = 'pileupName'
        try:
            MSPileupObj(wrongData, validRSEs=expectedRSEs)
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # change replicationGrouping to bizzare value
        wrongData = dict(data)
        wrongData['replicationGrouping'] = 'bla'
        try:
            MSPileupObj(wrongData, validRSEs=expectedRSEs)
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

    def testTransitionRecord(self):
        """
        Unit test for transition record in MSPileupObj data
        """
        expectedRSEs = ['rse1', 'rse2']
        fullReplicas = 0
        campaigns = ['c1', 'c2']
        data = {
            'pileupName': '/lksjdfkls/lkjaslkdjlas/PREFIX',
            'pileupType': 'classic',
            'expectedRSEs': expectedRSEs,
            'currentRSEs': expectedRSEs,
            'fullReplicas': fullReplicas,
            'campaigns': campaigns,
            'containerFraction': 1.0,
            'replicationGrouping': "ALL",
            'active': True,
            'pileupSize': 0,
            'customName': '',
            'transition': [{'bla': 1}],
            'ruleIds': []}
        # test incorrect transition record
        try:
            MSPileupObj(data, validRSEs=expectedRSEs)
            self.assertIsNone(1, msg="MSPileupObj should not be created")
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # test incorrect value in transition record
        trec = {'DN': 'bla', 'containerFraction': 0, 'customDID': 'customDID', 'updateTime': int(time.time())}
        data.update({'transition': [trec]})
        try:
            MSPileupObj(data, validRSEs=expectedRSEs)
            self.assertIsNone(1, msg="MSPileupObj should not be created")
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # test incorrect value in transition record
        trec = {'DN': 123, 'containerFraction': 0.1, 'customDID': 'customDID', 'updateTime': int(time.time())}
        data.update({'transition': [trec]})
        try:
            MSPileupObj(data, validRSEs=expectedRSEs)
            self.assertIsNone(1, msg="MSPileupObj should not be created")
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # test incorrect value in transition record
        trec = {'DN': 'dn', 'containerFraction': 0.1, 'customDID': 'customDID', 'updateTime': 123}
        data.update({'transition': [trec]})
        try:
            MSPileupObj(data, validRSEs=expectedRSEs)
            self.assertIsNone(1, msg="MSPileupObj should not be created")
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

        # test correct record
        trec = {'DN': 'bla', 'containerFraction': 0.5, 'customDID': 'customDID', 'updateTime': int(time.time())}
        data.update({'transition': [trec]})
        try:
            MSPileupObj(data, validRSEs=expectedRSEs)
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))

    def testWrongMSPileupObj(self):
        """
        Unit test for incorrect MSPileupObj data
        """
        data = {
            'pileupName': 'pileupName',
            'pileupType': 1,
            'expectedRSEs': 'expectedRSEs',
            'fullReplicas': 'fullReplicas',
            'campaigns': 'campaigns',
            'containerFraction': 1,
            'replicationGrouping': "",
            'active': True,
            'pileupSize': 0,
            'ruleIds': []}
        try:
            MSPileupObj(data)
            self.assertIsNone(1, msg="MSPileupObj should not be created")
        except Exception as exp:
            self.logger.warning("expected exception %s", str(exp))


if __name__ == '__main__':
    unittest.main()



"""
Unit tests for Unified/Transferor.py module

Author: Valentin Kuznetsov <vkuznet [AT] gmail [DOT] com>
"""
from __future__ import division, print_function

import json
# system modules
import os
import unittest

# WMCore modules
from Utils.PythonVersion import PY3
from WMCore.MicroService.MSTransferor.MSTransferor import MSTransferor
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMCore.MicroService.MSTransferor.MSTransferorError import MSTransferorStorageError
from WMCore.MicroService.MSTransferor.DataStructs.Workflow import Workflow


def getTestFile(partialPath):
    """
    Returns the absolute path for the test json file
    """
    normPath = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..'))
    return os.path.join(normPath, partialPath)


class TransferorTest(EmulatedUnitTestCase):
    "Unit test for Transferor module"

    def setUp(self):
        "init test class"
        self.msConfig = {'services': ['transferor'],
                         'verbose': False,
                         'interval': 1 * 60,
                         'enableStatusTransition': True,
                         'enableDataTransfer': False,
                         'reqmgr2Url': 'https://cmsweb-testbed.cern.ch/reqmgr2',
                         'reqmgrCacheUrl': 'https://cmsweb-testbed.cern.ch/couchdb/reqmgr_workload_cache',
                         'quotaUsage': 0.9,
                         'rucioAccount': 'wma_test',  # it should be wmcore_transferor
                         # 'rucioAuthUrl': 'https://cms-rucio-auth.cern.ch',
                         # 'rucioUrl': 'http://cms-rucio.cern.ch',
                         'rucioAuthUrl': 'https://cms-rucio-auth-int.cern.ch',
                         'rucioUrl': 'http://cms-rucio-int.cern.ch',
                         'dbsUrl': 'https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader'}

        self.msTransferor = MSTransferor(self.msConfig)

        self.taskChainTempl = getTestFile('data/ReqMgr/requests/Integration/TaskChain_Prod.json')
        self.stepChainTempl = getTestFile('data/ReqMgr/requests/Integration/SC_LumiMask_PhEDEx.json')
        super(TransferorTest, self).setUp()
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testGetPNNsFromPSNs(self):
        """Test MSTransferor private method _getPNNsFromPSNs()"""
        self.assertItemsEqual(self.msTransferor.psn2pnnMap, {})

        # now fill up the cache
        self.msTransferor.psn2pnnMap = self.msTransferor.cric.PSNtoPNNMap()

        self.assertItemsEqual(self.msTransferor._getPNNsFromPSNs([]), set())
        pnns = self.msTransferor._getPNNsFromPSNs(["T1_IT_CNAF", "T1_IT_CNAF_Disk"])
        self.assertItemsEqual(pnns, set(["T1_IT_CNAF_Disk"]))

        # dropping T3s and CERNBOX
        pnns = self.msTransferor._getPNNsFromPSNs(["T1_US_FNAL", "T2_CH_CERN_HLT"])
        self.assertItemsEqual(pnns, set(["T1_US_FNAL_Disk", "T2_CH_CERN"]))

        # testing with non-existant PSNs
        psns = self.msTransferor._getPNNsFromPSNs(["T1_US_FNAL_Disk", "T2_CH_CERNBOX"])
        self.assertItemsEqual(psns, set())

    def testGetPSNsFromPNNs(self):
        """Test MSTransferor private method _getPSNsFromPNNs()"""
        self.assertItemsEqual(self.msTransferor.pnn2psnMap, {})

        # now fill up the cache
        self.msTransferor.pnn2psnMap = self.msTransferor.cric.PNNtoPSNMap()

        self.assertItemsEqual(self.msTransferor._getPSNsFromPNNs([]), set())
        psns = self.msTransferor._getPSNsFromPNNs(["T1_IT_CNAF", "T1_IT_CNAF_Disk"])
        self.assertItemsEqual(psns, set(["T1_IT_CNAF"]))

        # test dropping T3s
        psns = self.msTransferor._getPSNsFromPNNs(["T2_UK_SGrid_RALPP"])
        self.assertItemsEqual(psns, set(["T2_UK_SGrid_RALPP"]))

        # testing with non-existant PNNs
        psns = self.msTransferor._getPSNsFromPNNs(["T1_US_FNAL", "T2_CH_CERN_HLT"])
        self.assertItemsEqual(psns, set())

    def testDiskPNNs(self):
        """Test MSTransferor private method _diskPNNs()"""
        # empty list of pnns
        self.assertItemsEqual(self.msTransferor._diskPNNs([]), set())

        # only PNNs that will be dropped
        pnns = self.msTransferor._diskPNNs(["T1_US_FNAL_Tape", "T1_US_FNAL_MSS",
                                            "T2_CH_CERNBOX", "T0_CH_CERN_Export"])
        self.assertItemsEqual(pnns, set())

        # valid PNNs that can receive data
        pnns = self.msTransferor._diskPNNs(["T1_US_FNAL_Disk", "T2_CH_CERN", "T2_DE_DESY"])
        self.assertItemsEqual(pnns, set(["T1_US_FNAL_Disk", "T2_CH_CERN", "T2_DE_DESY"]))

        # finally, a mix of valid and invalid PNNs
        pnns = self.msTransferor._diskPNNs(["T1_US_FNAL_Disk", "T1_US_FNAL_MSS", "T1_US_FNAL_Tape",
                                            "T2_CH_CERN", "T2_DE_DESY"])
        self.assertItemsEqual(pnns, set(["T1_US_FNAL_Disk", "T2_CH_CERN", "T2_DE_DESY"]))

    def notestRequestRecord(self):
        """
        Test the requestRecord method
        """
        default = {'name': '', 'reqStatus': None, 'SiteWhiteList': [],
                   'SiteBlackList': [], 'datasets': [], 'campaign': []}
        self.assertItemsEqual(self.msTransferor.requestRecord({}), default)

        with open(self.taskChainTempl) as jo:
            reqData = json.load(jo)['createRequest']
        expectedRes = [{'type': 'MCPileup',
                        'name': '/Neutrino_E-10_gun/RunIISummer17PrePremix-PUAutumn18_102X_upgrade2018_realistic_v15-v1/GEN-SIM-DIGI-RAW'},
                       {'type': 'MCPileup',
                        'name': '/Neutrino_E-10_gun/RunIISummer17PrePremix-PUAutumn18_102X_upgrade2018_realistic_v15-v1/GEN-SIM-DIGI-RAW'}]
        resp = self.msTransferor.requestRecord(reqData)['datasets']
        self.assertEqual(len(resp), 2)
        for idx in range(len(resp)):
            self.assertItemsEqual(resp[idx], expectedRes[idx])

        with open(self.stepChainTempl) as jo:
            reqData = json.load(jo)['createRequest']
        expectedRes = [{'type': 'InputDataset',
                        'name': '/RelValH125GGgluonfusion_14/CMSSW_10_6_1-106X_mcRun3_2021_realistic_v1_rsb-v1/GEN-SIM'},
                       {'type': 'MCPileup',
                        'name': '/RelValMinBias_14TeV/CMSSW_10_6_1-106X_mcRun3_2021_realistic_v1_rsb-v1/GEN-SIM'},
                       {'type': 'MCPileup',
                        'name': '/RelValMinBias_14TeV/CMSSW_10_6_1-106X_mcRun3_2021_realistic_v1_rsb-v1/GEN-SIM'}]
        resp = self.msTransferor.requestRecord(reqData)['datasets']
        self.assertEqual(len(resp), 3)
        for idx in range(len(resp)):
            self.assertItemsEqual(resp[idx], expectedRes[idx])

    def testUpdateStorage(self):
        """
        Test updateStorage method. We should be able to save and read
        JSON objects to persistent storage of MSTransferor.
        """
        # use default storage and check save/read operations
        wflow = 'testWorkflow'
        status = self.msTransferor.updateStorage(wflow)
        self.assertEqual(status, 'ok')
        wflowObject = Workflow(wflow, {'DbsUrl': 'https://cmsweb-testbed.cern.ch', 'RequestType': 'StoreResults'})
        self.msTransferor.checkDataReplacement(wflowObject)
        self.assertEqual(wflowObject.dataReplacement, True)

    def testUpdateSites(self):
        """
        Test the updateSites method.
        """
        rec = {'workflow': 'testWorkflow'}
        res = self.msTransferor.updateSites(rec)
        self.assertEqual(res, [])

        # now let's test transferor error
        self.msTransferor.storage = '/bla'
        res = self.msTransferor.updateSites(rec)
        err = MSTransferorStorageError("[Errno 2] No such file or directory: '/bla/testWorkflow'", **rec)
        self.assertEqual(res, [err.error()])
        self.assertEqual(err.code, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_LoadBlocksByDAS_

Oracle implementation of LoadBlocksByDAS
"""




from WMComponent.DBS3Buffer.MySQL.LoadBlocksByDAS import LoadBlocksByDAS as MySQLLoadBlocksByDAS

class LoadBlocksByDAS(MySQLLoadBlocksByDAS):
    """
    _LoadBlocksByDAS_

    Untested; same as MySQL
    """



"""
"""


import logging
import os
from WMCore.Configuration import loadConfigurationFile

def _getConfiguration():
    configFile = os.environ.get("EMULATOR_CONFIG")
    if configFile:
        return loadConfigurationFile(configFile)
    return None

def emulatorSwitch(name):
    try:
        config = _getConfiguration()
        if config:
            if getattr(config.Emulator, name, False):
                logging.info("Using %s Emulator ..." %  name)
                return True
        return False

    except Exception as e:
        logging.warning("Emulator Config is not set:  %s"  % str(e))
        return False



#!/usr/bin/env python
"""
_Registry_

Registry for StageOutImpl derived plugins

Note that this is a runtime module, so provides its own registry
rather than using the common implementation to avoid generating
extra runtime dependencies.

"""
from builtins import object
import WMCore.WMFactory
from WMCore.Storage.StageOutError import StageOutError
from WMCore.Storage.StageOutImpl import StageOutImpl


class RegistryError(StageOutError):
    """
    _RegistryError_

    Error class for handling registry issues
    """
    pass


class Registry(object):
    """
    _Registry_

    Singleton namespace for storing static reference to all
    stage out impl class objects

    """
    StageOutImpl = {}

    def __init__(self):
        msg = "Do not init StageOut.Registry class"
        raise RuntimeError(msg)


def registerStageOutImpl(name, classRef):
    """
    _registerStageOutImpl_

    Register a StageOutImpl subclass with the name provided

    """
    if name in Registry.StageOutImpl:
        msg = "Duplicate StageOutImpl registered for name: %s\n" % name
        raise RegistryError(msg)

    if not issubclass(classRef, StageOutImpl):
        msg = "StageOutImpl object registered as %s\n" % name
        msg += "is not a subclass of StageOut.StageOutImpl\n"
        msg += "Registration should be of a class that inherits StageOutImpl"
        raise RegistryError(msg)

    Registry.StageOutImpl[name] = classRef
    return


def retrieveStageOutImpl(name, stagein=False, useNewVersion=False):
    """
    _retrieveStageOutImpl_

    Get the matching impl class and return an instance of it

    """
    if not useNewVersion:
        classRef = Registry.StageOutImpl.get(name, None)
    else:
        try:
            return _retrieveStageOutImpl2(name)
        except ImportError:
            raise RegistryError("Stageout plugin %s doesn't exist" % name)

    if not useNewVersion:
        return classRef(stagein)
    else:
        return classRef()


pluginLookup = {'test-win': 'TestWinImpl',
                'test-fail': 'TestFailImpl',
                'test-copy': 'TestLocalCopyImpl',
                "cp": "CPImpl",
                "stageout-xrdcp-fnal": 'FNALImpl',
                "srmv2": 'SRMV2Impl',
                "srmv2-lcg": 'LCGImpl',
                "xrdcp": 'XRDCPImpl',
                "vandy": 'VandyImpl',
                "gfal2": 'GFAL2Impl',
                "awss3": 'AWS3Impl',
                # NOTE NOTE NOTE:
                # do NOT implement this
                "testFallbackToOldBackend": 'TestBackendForFallbacksDontImplement'}


def _retrieveStageOutImpl2(backendName):
    factory = WMCore.WMFactory.WMFactory(name='StageOutFactory',
                                         namespace='WMCore.Storage.Plugins')
    className = pluginLookup[backendName]
    stageout = factory.loadObject(className)
    return stageout



#!/usr/bin/env python
"""
_SetBlockFiles_

Oracle implementation of DBS3Buffer.SetBlockFiles
"""

from WMComponent.DBS3Buffer.MySQL.SetBlockFiles import SetBlockFiles as MySQLSetBlockFiles

class SetBlockFiles(MySQLSetBlockFiles):
    pass



#!/usr/bin/env python
"""
_GetSubTypes_

Oracle implementation of Jobs.GetSubTypes
"""

__all__ = []



import logging

from WMCore.WMBS.MySQL.Subscriptions.GetSubTypes import GetSubTypes as MySQLGetSubTypes



class GetSubTypes(MySQLGetSubTypes):
    """
    Identical to MySQL version

    """



import logging

class TestRegistryFile1:
    def __init__(self, **parameters):
        logging.debug("Instantitating object with parameters:")
        logging.debug(str(parameters))

    def doSomething(self, parameters = {}):
        logging.debug("TestCollection_3.TestRegistryFile1 is doing something")
        logging.debug("With parameters: "+str(parameters))
        return parameters



from __future__ import division, print_function

import time

from builtins import object
from future.utils import viewitems

import logging

from Utils.IteratorTools import nestedDictUpdate, grouper
from WMCore.Database.CMSCouch import CouchServer
from WMCore.Lexicon import splitCouchServiceURL, sanitizeURL
from WMCore.Services.RequestDB.RequestDBReader import RequestDBReader
from WMCore.Services.WMStats.DataStruct.RequestInfoCollection import RequestInfo
from WMCore.ReqMgr.DataStructs.RequestStatus import T0_ACTIVE_STATUS
from WMCore.Services.WMStats.WMStatsPycurl import getTaskJobSummaryByRequestPycurl


REQUEST_PROPERTY_MAP = {
    "_id": "_id",
    "InputDataset": "inputdataset",
    "PrepID": "prep_id",
    "Group": "group",
    "RequestDate": "request_date",
    "Campaign": "campaign",
    "RequestName": "workflow",
    "RequestorDN": "user_dn",
    "RequestPriority": "priority",
    "Requestor": "requestor",
    "RequestType": "request_type",
    "DbsUrl": "dbs_url",
    "CMSSWVersion": "cmssw",
    "OutputDatasets": "outputdatasets",
    "RequestTransition": "request_status",  # Status: status,  UpdateTime: update_time
    "SiteWhitelist": "site_white_list",
    "Team": "team",
    "TotalEstimatedJobs": "total_jobs",
    "TotalInputEvents": "input_events",
    "TotalInputLumis": "input_lumis",
    "TotalInputFiles": "input_num_files",
    "Run": "run",
    # Status and UpdateTime is under "RequestTransition"
    "Status": "status",
    "UpdateTime": "update_time"
}


def convertToLegacyFormat(requestDoc):
    converted = {}
    for key, value in viewitems(requestDoc):

        if key == "RequestTransition":
            newValue = []
            for transDict in value:
                newItem = {}
                for transKey, transValue in viewitems(transDict):
                    newItem[REQUEST_PROPERTY_MAP.get(transKey, transKey)] = transValue
                    newValue.append(newItem)
            value = newValue

        converted[REQUEST_PROPERTY_MAP.get(key, key)] = value

    return converted


class WMStatsReader(object):

    def __init__(self, couchURL, appName="WMStats", reqdbURL=None,
                 reqdbCouchApp="ReqMgr", logger=None):
        self._sanitizeURL(couchURL)
        # set the connection for local couchDB call
        self._commonInit(couchURL, appName)
        if reqdbURL:
            self.reqDB = RequestDBReader(reqdbURL, reqdbCouchApp)
        else:
            self.reqDB = None
        self.logger = logger if logger else logging.getLogger()
        self.serviceOpts = {"couchURL": self.couchURL,
                            "dbName": self.dbName,
                            "couchapp": self.couchapp}

    def _sanitizeURL(self, couchURL):
        return sanitizeURL(couchURL)['url']

    def _commonInit(self, couchURL, appName="WMStats"):
        """
        setting up comon variables for inherited class.
        inherited class should call this in their init function
        """

        self.couchURL, self.dbName = splitCouchServiceURL(couchURL)
        self.couchServer = CouchServer(self.couchURL)
        self.couchDB = self.couchServer.connectDatabase(self.dbName, False)
        self.couchapp = appName
        self.defaultStale = {"stale": "update_after"}

    def setDefaultStaleOptions(self, options):
        if not options:
            options = {}
        if 'stale' not in options:
            options.update(self.defaultStale)
        return options

    def getLatestJobInfoByRequests(self, requestNames):
        jobInfoByRequestAndAgent = {}

        if len(requestNames) > 0:
            requestAndAgentKey = self._getRequestAndAgent(requestNames)
            jobInfoByRequestAndAgent = self._getLatestJobInfo(requestAndAgentKey)
        return jobInfoByRequestAndAgent

    def _updateRequestInfoWithJobInfo(self, requestInfo):
        if requestInfo:
            jobInfoByRequestAndAgent = self.getLatestJobInfoByRequests(list(requestInfo))
            self._combineRequestAndJobData(requestInfo, jobInfoByRequestAndAgent)

    def _getCouchView(self, view, options, keys=None):
        keys = keys or []
        options = self.setDefaultStaleOptions(options)

        if keys and isinstance(keys, str):
            keys = [keys]
        return self.couchDB.loadView(self.couchapp, view, options, keys)

    def _formatCouchData(self, data, key="id"):
        result = {}
        for row in data['rows']:
            if 'error' in row:
                continue
            if "doc" in row:
                result[row[key]] = row["doc"]
            else:
                result[row[key]] = None
        return result

    def _combineRequestAndJobData(self, requestData, jobData):
        """
        update the request data with job info
        requestData['AgentJobInfo'] = {'vocms234.cern.ch:9999': {"_id":"d1d11dfcb30e0ab47db42007cb6fb847",
        "_rev":"1-8abfaa2de822ed081cb8d174e3e2c003",
        "status":{"inWMBS":334,"success":381,"submitted":{"retry":2,"pending":2},"failure":{"exception":3}},
        "agent_team":"testbed-integration","workflow":"amaltaro_OracleUpgrade_TEST_HG1401_140220_090116_6731",
        "timestamp":1394738860,"sites":{"T2_CH_CERN_AI":{"submitted":{"retry":1,"pending":1}},
        "T2_CH_CERN":{"success":6,"submitted":{"retry":1,"pending":1}},
        "T2_DE_DESY":{"failure":{"exception":3},"success":375}},
        "agent":"WMAgent",
        "tasks":
           {"/amaltaro_OracleUpgrade_TEST_HG1401_140220_090116_6731/Production":
            {"status":{"failure":{"exception":3},"success":331},
             "sites":{"T2_DE_DESY": {"success":325,"wrappedTotalJobTime":11305908,
                                     "dataset":{},"failure":{"exception":3},
                                     "cmsRunCPUPerformance":{"totalJobCPU":10869688.8,
                                                             "totalEventCPU":10832426.7,
                                                             "totalJobTime":11255865.9},
                                     "inputEvents":0},
                      "T2_CH_CERN":{"success":6,"wrappedTotalJobTime":176573,
                                    "dataset":{},
                                    "cmsRunCPUPerformance":{"totalJobCPU":167324.8,
                                                            "totalEventCPU":166652.1,
                                                            "totalJobTime":174975.7},
                                    "inputEvents":0}},
             "subscription_status":{"updated":1393108089, "finished":2, "total":2,"open":0},
             "jobtype":"Production"},
            "/amaltaro_OracleUpgrade_TEST_HG1401_140220_090116_6731/Production/ProductionMergeRAWSIMoutput/ProductionRAWSIMoutputMergeLogCollect":
             {"jobtype":"LogCollect",
              "subscription_status":{"updated":1392885768,
              "finished":0,
              "total":1,"open":1}},
            "/amaltaro_OracleUpgrade_TEST_HG1401_140220_090116_6731/Production/ProductionMergeRAWSIMoutput":
              {"status":{"success":41,"submitted":{"retry":1,"pending":1}},
                "sites":{"T2_DE_DESY":{"datasetStat":{"totalLumis":973,"events":97300,"size":105698406915},
                                       "success":41,"wrappedTotalJobTime":9190,
                                       "dataset":{"/GluGluToHTohhTo4B_mH-350_mh-125_8TeV-pythia6-tauola/Summer12-OracleUpgrade_TEST_ALAN_HG1401-v1/GEN-SIM":
                                                   {"totalLumis":973,"events":97300,"size":105698406915}},
                                       "cmsRunCPUPerformance":{"totalJobCPU":548.92532,"totalEventCPU":27.449808,"totalJobTime":2909.92125},
                                    "inputEvents":97300},
                         "T2_CH_CERN":{"submitted":{"retry":1,"pending":1}}},
                "subscription_status":{"updated":1392885768,"finished":0,"total":1,"open":1},
                "jobtype":"Merge"},
           "agent_url":"vocms231.cern.ch:9999",
           "type":"agent_request"}}
        """
        if jobData:
            for row in jobData["rows"]:
                # condition checks if documents are deleted between calls.
                # just ignore in that case
                if row["doc"]:
                    jobInfo = requestData[row["doc"]["workflow"]]
                    jobInfo.setdefault("AgentJobInfo", {})
                    jobInfo["AgentJobInfo"][row["doc"]["agent_url"]] = row["doc"]

    def _getRequestAndAgent(self, filterRequest=None):
        """
        returns the [['request_name', 'agent_url'], ....]
        """
        options = {}
        options["reduce"] = True
        options["group"] = True
        result = self._getCouchView("requestAgentUrl", options)

        if filterRequest is None:
            keys = [row['key'] for row in result["rows"]]
        else:
            keys = [row['key'] for row in result["rows"] if row['key'][0] in filterRequest]
        return keys

    def _getLatestJobInfo(self, keys):
        """
        Given a list of lists as keys, in the format of:
            [['request_name', 'agent_url'], ['request_name2', 'agent_url2'], ....]
        The result format from the latestRequest view is:
            {u'offset': 527,
             u'rows': [{u'doc': {u'_rev': u'32-6027014210',
             ...
                        u'id': u'cmsgwms-submit6.fnal.gov-cmsunified_ACDC0_task_BTV-RunIISummer19UL18wmLHEGEN-00004__v1_T_200507_162125_3670',
                        u'key': [u'cmsunified_ACDC0_task_BTV-RunIISummer19UL18wmLHEGEN-00004__v1_T_200507_162125_3670',
                                 u'cmsgwms-submit6.fnal.gov'],
                        u'value': None}],
             u'total_rows': 49606}
        """
        if not keys:
            return []
        options = {}
        options["include_docs"] = True
        options["reduce"] = False
        finalResults = {}
        # magic number: 5000 keys (need to check which number is optimal)
        for sliceKeys in grouper(keys, 5000):
            self.logger.info("Querying latestRequest with %d keys", len(sliceKeys))
            result = self._getCouchView("latestRequest", options, sliceKeys)
            if not finalResults and result:
                finalResults = result
            elif result.get('rows'):
                finalResults['rows'].extend(result['rows'])
        return finalResults

    def _getAllDocsByIDs(self, ids, include_docs=True):
        """
        keys is [id, ....]
        returns document
        """
        if len(ids) == 0:
            return None
        options = {}
        options["include_docs"] = include_docs
        result = self.couchDB.allDocs(options, ids)

        return result

    def _getAgentInfo(self):
        """
        returns all the agents status on wmstats
        """
        options = {}
        result = self._getCouchView("agentInfo", options)

        return result

    def agentsByTeam(self, filterDrain=False):
        """
        return a dictionary like {team:#agents,...}
        """
        result = self._getAgentInfo()
        response = dict()

        for agentInfo in result["rows"]:
            #filtering empty string
            team = agentInfo['value']['agent_team']
            if not team:
                continue

            response.setdefault(team, 0)
            if filterDrain:
                if not agentInfo['value'].get('drain_mode', False):
                    response[team] += 1
            else:
                response[team] += 1

        return response

    def getServerInstance(self):
        return self.couchServer

    def getDBInstance(self):
        return self.couchDB

    def getRequestDBInstance(self):
        return self.reqDB

    def getHeartbeat(self):
        try:
            return self.couchDB.info()
        except Exception as ex:
            return {'error_message': str(ex)}

    def getRequestByNames(self, requestNames, jobInfoFlag=False):
        """
        To use this function reqDBURL need to be set when wmstats initialized.
        This will be deplicated so please don use this.
        """
        requestInfo = self.reqDB.getRequestByNames(requestNames, True)

        if jobInfoFlag:
            # get request and agent info
            self._updateRequestInfoWithJobInfo(requestInfo)
        return requestInfo

    def getActiveData(self, listStatuses, jobInfoFlag=False):
        return self.getRequestByStatus(listStatuses, jobInfoFlag)

    def getT0ActiveData(self, jobInfoFlag=False):

        return self.getRequestByStatus(T0_ACTIVE_STATUS, jobInfoFlag)

    def getRequestByStatus(self, statusList, jobInfoFlag=False, limit=None, skip=None,
                           legacyFormat=False):

        """
        To use this function reqDBURL need to be set when wmstats initialized.
        This will be deplicated so please don use this.
        If legacyFormat is True convert data to old wmstats format from current reqmgr format.
        Shouldn't be set to True unless existing code breaks
        """
        results = dict()
        for status in statusList:
            self.logger.info("Fetching workflows by status from ReqMgr2, status: %s", status)
            requestInfo = self.reqDB.getRequestByStatus(status, True, limit, skip)
            self.logger.info("Found %d workflows in status: %s", len(requestInfo), status)

            if legacyFormat:
                # convert the format to wmstats old format
                for requestName, doc in viewitems(requestInfo):
                    requestInfo[requestName] = convertToLegacyFormat(doc)
            results.update(requestInfo)

        # now update these requests with agent information too
        if results and jobInfoFlag:
            self.logger.info("Now updating these requests with job info...")
            self._updateRequestInfoWithJobInfo(results)

        return results

    def getRequestSummaryWithJobInfo(self, requestName):
        """
        get request info with job status
        """
        requestInfo = self.reqDB.getRequestByNames(requestName)
        self._updateRequestInfoWithJobInfo(requestInfo)
        return requestInfo

    def getArchivedRequests(self):
        """
        get list of archived workflow in wmstats db.
        """

        options = {"group_level": 1, "reduce": True}

        results = self._getCouchView("allWorkflows", options)['rows']
        requestNames = [x['key'] for x in results]

        workflowDict = self.reqDB.getStatusAndTypeByRequest(requestNames)
        archivedRequests = []
        for request, value in viewitems(workflowDict):
            if value[0].endswith("-archived"):
                archivedRequests.append(request)

        return archivedRequests

    def isWorkflowCompletedWithLogCollectAndCleanUp(self, requestName):
        """
        check whether workflow  is completed including LogCollect and CleanUp tasks
        TODO: If the parent task all failed and next task are not created at all,
            It can't detect complete status.
            If the one of the task doesn't contain any jobs, it will return False
        """

        requestInfo = self.getRequestSummaryWithJobInfo(requestName)
        reqInfoInstance = RequestInfo(requestInfo[requestName])
        return reqInfoInstance.isWorkflowFinished()

    def getTaskJobSummaryByRequest(self, requestName, sampleSize=1, usePycurl=True):
        reqStart = time.time()
        options = {'reduce': True, 'group_level': 5, 'startkey': [requestName],
                   'endkey': [requestName, {}]}
        results = self._getCouchView("jobsByStatusWorkflow", options)

        jobDetails = {}
        if usePycurl is True:
            jobDetails = getTaskJobSummaryByRequestPycurl(results, sampleSize, self.serviceOpts)
        else:
            # then it is sequential
            for row in results['rows']:
                # row["key"] = ['workflow', 'task', 'jobstatus', 'exitCode', 'site']
                startKey = row["key"][:4]
                endKey = []
                site = row["key"][4]
                if site:
                    startKey.append(site)

                endKey.extend(startKey)
                endKey.append({})
                numOfError = row["value"]

                jobInfo = self.jobDetailByTasks(startKey, endKey, numOfError, sampleSize)
                jobDetails = nestedDictUpdate(jobDetails, jobInfo)
        callRuntime = round((time.time() - reqStart), 3)
        msg = f"Retrieved job details (pycurl mode: {usePycurl}) for {requestName} in "
        msg += f"{callRuntime} seconds, with a total of {len(results['rows']) + 1} CouchDB calls"
        print(msg)
        return jobDetails

    def jobDetailByTasks(self, startKey, endKey, numOfError, limit=1):
        options = {'include_docs': True, 'reduce': False,
                   'startkey': startKey, 'endkey': endKey,
                   'limit': limit}
        result = self._getCouchView("jobsByStatusWorkflow", options)
        jobInfoDoc = {}
        for row in result['rows']:
            keys = row['key']
            workflow = keys[0]
            task = keys[1]
            jobStatus = keys[2]
            exitCode = keys[3]
            site = keys[4]

            jobInfoDoc.setdefault(workflow, {})
            jobInfoDoc[workflow].setdefault(task, {})
            jobInfoDoc[workflow][task].setdefault(jobStatus, {})
            jobInfoDoc[workflow][task][jobStatus].setdefault(exitCode, {})
            jobInfoDoc[workflow][task][jobStatus][exitCode].setdefault(site, {})
            finalStruct = jobInfoDoc[workflow][task][jobStatus][exitCode][site]
            finalStruct["errorCount"] = numOfError
            finalStruct.setdefault("samples", [])
            finalStruct["samples"].append(row["doc"])

        return jobInfoDoc

    def getAllAgentRequestRevByID(self, agentURL):
        options = {"reduce": False}
        results = self._getCouchView("byAgentURL", options, keys=[agentURL])
        idRevMap = {}
        for row in results['rows']:
            idRevMap[row['id']] = row['value']['rev']

        return idRevMap



"""
Unit tests for the WMCore/MicroService/DataStructs/RelValWorkflow module
"""
from __future__ import division, print_function

import unittest
from copy import deepcopy

from WMCore.MicroService.MSTransferor.DataStructs.RelValWorkflow import RelValWorkflow


class RelValWorkflowTest(unittest.TestCase):
    """
    Test the very basic functionality of the RelValWorkflow module
    """

    def setUp(self):
        """
        Defined some basic data structs to use in the unit tests
        :return: None
        """
        self.primaryDict = {"block_A": {"blockSize": 1, "locations": ["Site_A", "Site_B"]},
                            "block_B": {"blockSize": 2, "locations": ["Site_B", "Site_C"]}}
        self.parentDict = {"parent_A": {"blockSize": 11, "locations": ["Site_A"]},
                           "parent_B": {"blockSize": 12, "locations": ["Site_B"]}}
        self.relvalSpec = {"RequestType": "TaskChain",
                           "SubRequestType": "RelVal",
                           "TaskChain": 3,
                           "Campaign": "top-campaign",
                           "RequestName": "whatever_name",
                           "DbsUrl": "a_dbs_url",
                           "SiteWhitelist": ["Site_A", "Site_B", "Site_C"],
                           "SiteBlacklist": [],
                           "Task1": {"Campaign": "task1-campaign"},
                           "Task2": {"Campaign": "task2-campaign"},
                           "Task3": {"Campaign": "task3-campaign"},
                           }
        self.relvalNoInputPU = {"Task1": {"Campaign": "task1-campaign"},
                                "Task2": {"MCPileup": "/task1/mc-pileup/tier",
                                          "Campaign": "task2-campaign"},
                                "Task3": {"Campaign": "task3-campaign"}}
        self.relvalInputNoPU = {"Task1": {"InputDataset": "/task1/input-dataset/tier",
                                          "Campaign": "task1-campaign"},
                                "Task2": {"Campaign": "task2-campaign"},
                                "Task3": {"Campaign": "task3-campaign"}}
        self.relvalInputPU = {"Task1": {"InputDataset": "/task1/input-dataset/tier",
                                        "Campaign": "task1-campaign"},
                              "Task2": {"MCPileup": "/task2/mc-pileup/tier",
                                        "Campaign": "task2-campaign"},
                              "Task3": {"Campaign": "task3-campaign"}}
        self.relvalInputDualPU = {"Task1": {"InputDataset": "/task1/input-dataset/tier",
                                            "Campaign": "task1-campaign"},
                                  "Task2": {"MCPileup": "/task2/mc-pileup/tier",
                                            "Campaign": "task2-campaign"},
                                  "Task3": {"MCPileup": "/task3/mc-pileup/tier",
                                            "Campaign": "task3-campaign"}}

    def tearDown(self):
        pass

    def testInstance(self):
        """
        Test object instance type
        """
        wflow = RelValWorkflow(self.relvalSpec['RequestName'], deepcopy(self.relvalSpec))
        self.assertIsInstance(wflow, RelValWorkflow)

    def testRelValWflowNoInputNoPU(self):
        """
        Test loading a RelVal like request without any input and pileup
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        wflow = RelValWorkflow(self.relvalSpec['RequestName'], deepcopy(self.relvalSpec))
        self.assertEqual(wflow.getName(), self.relvalSpec['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), self.relvalSpec['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), self.relvalSpec["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), self.relvalSpec["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), set())
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)

    def testRelValWflowNoInputWithPU(self):
        """
        Test loading a RelVal like request without input but with pileup
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalNoInputPU))

        wflow = RelValWorkflow(self.relvalSpec['RequestName'], specDict)
        self.assertEqual(wflow.getName(), specDict['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), specDict['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), specDict["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), specDict["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), [specDict["Task2"].get("MCPileup", "")])
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)

    def testRelValWflowInputNoPU(self):
        """
        Test loading a RelVal like request with input but no pileup
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalInputNoPU))

        wflow = RelValWorkflow(self.relvalSpec['RequestName'], specDict)
        self.assertEqual(wflow.getName(), specDict['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), specDict['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), specDict["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), specDict["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), [])
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)

    def testRelValWflowInputPU(self):
        """
        Test loading a RelVal like request with input and pileup
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalInputPU))
        puName = specDict["Task2"].get("MCPileup", "")

        wflow = RelValWorkflow(self.relvalSpec['RequestName'], specDict)
        self.assertEqual(wflow.getName(), specDict['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), specDict['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), specDict["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), specDict["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), [puName])
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(wflow.getPrimaryBlocks(), {})

        # checking pileup data
        self.assertEqual(wflow.getSecondarySummary(), {})
        wflow.setSecondarySummary(puName, locations=["Site_B"])
        self.assertCountEqual(list(wflow.getSecondarySummary()), [puName])
        self.assertCountEqual(wflow.getSecondarySummary()[puName]["locations"], ["Site_B"])

    def testRelValWflowWithParent(self):
        """
        Test loading a RelVal like request with input and parent data
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalInputNoPU))
        specDict["Task1"].update(dict(IncludeParents=True))

        wflow = RelValWorkflow(self.relvalSpec['RequestName'], specDict)
        self.assertEqual(wflow.getName(), specDict['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), specDict['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), specDict["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), specDict["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), [])
        self.assertTrue(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(wflow.getPrimaryBlocks(), {})

        wflow.setParentBlocks(self.parentDict)
        wflow.setPrimaryBlocks(self.primaryDict)
        self.assertCountEqual(list(wflow.getPrimaryBlocks()), ["block_A", "block_B"])
        self.assertCountEqual(list(wflow.getParentBlocks()), ["parent_A", "parent_B"])

    def testRelValWflowDualPU(self):
        """
        Test loading a RelVal like request with input and two pileups
        """
        expCampaigns = {'task2-campaign', 'task1-campaign', 'task3-campaign'}
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalInputDualPU))
        puName1 = specDict["Task2"].get("MCPileup", "")
        puName2 = specDict["Task3"].get("MCPileup", "")

        wflow = RelValWorkflow(self.relvalSpec['RequestName'], specDict)
        self.assertEqual(wflow.getName(), specDict['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), specDict['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), specDict["SiteWhitelist"])
        self.assertCountEqual(wflow.getCampaigns(), expCampaigns)
        self.assertEqual(wflow.getInputDataset(), specDict["Task1"].get("InputDataset", ""))
        self.assertCountEqual(wflow.getPileupDatasets(), [puName1, puName2])
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(wflow.getPrimaryBlocks(), {})

        # checking pileup data
        self.assertEqual(wflow.getSecondarySummary(), {})
        wflow.setSecondarySummary(puName1, locations=["Site_B"])
        self.assertCountEqual(list(wflow.getSecondarySummary()), [puName1])
        self.assertCountEqual(wflow.getSecondarySummary()[puName1]["locations"], ["Site_B"])

        # now set the second pileup
        wflow.setSecondarySummary(puName2, locations=["Site_C"])
        self.assertCountEqual(list(wflow.getSecondarySummary()), [puName1, puName2])
        self.assertCountEqual(wflow.getSecondarySummary()[puName1]["locations"], ["Site_B"])
        self.assertCountEqual(wflow.getSecondarySummary()[puName2]["locations"], ["Site_C"])

    def testGetInputData(self):
        """
        Test the `getInputData` method for this template, which
        is supposed to return a list of input blocks
        """
        wflow = RelValWorkflow(self.relvalSpec['RequestName'], deepcopy(self.relvalSpec))
        wflow.setPrimaryBlocks(self.primaryDict)
        inputBlocks, blockSize = wflow.getInputData()
        self.assertEqual(len(inputBlocks), 2)
        self.assertCountEqual(inputBlocks, list(self.primaryDict))
        self.assertEqual(blockSize, 3)

    def testGetInputDataParent(self):
        """
        Test the `getInputData` method for this template, which
        is supposed to return a list of input blocks
        """
        specDict = deepcopy(self.relvalSpec)
        specDict.update(deepcopy(self.relvalInputNoPU))
        specDict["Task1"].update(dict(IncludeParents=True))
        wflow = RelValWorkflow(specDict['RequestName'], deepcopy(specDict))

        wflow.setPrimaryBlocks(self.primaryDict)
        wflow.setParentDataset("parent_dset")
        wflow.setParentBlocks(self.parentDict)

        inputBlocks, blockSize = wflow.getInputData()
        self.assertEqual(len(inputBlocks), 4)
        self.assertCountEqual(inputBlocks, list(self.primaryDict) + list(self.parentDict))
        self.assertEqual(blockSize, 26)

    def testGetRucioGrouping(self):
        """
        Test the `getRucioGrouping` method, which is supposed to return
        a basic string with the Rucio grouping for this template (static
        output).
        """
        wflow = RelValWorkflow(self.relvalSpec['RequestName'], deepcopy(self.relvalSpec))
        self.assertEqual(wflow.getRucioGrouping(), "ALL")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_WorkUnit_t_

Unittest for the WMCore.DataStructs.WorkUnit class

"""

from __future__ import absolute_import, division, print_function

import json
import unittest

from Utils.PythonVersion import PY3

from WMCore.DataStructs.Run import Run
from WMCore.DataStructs.WorkUnit import WorkUnit

TEST_TASKID = 1
TEST_LAST_UNIT_COUNT = 20
TEST_FILEID = 101
TEST_LUMI = 1
TEST_RUN_NUMBER = 1000000


class WorkUnitTest(unittest.TestCase):
    """
    _WorkUnitTest_
    """
    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testWorkUnitDefinitionDefault(self):
        """
        This tests the construction of a DataStructs WorkUnit object
        """

        testRunLumi = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                runLumi=testRunLumi)

        # Test the things we set
        self.assertEqual(testWorkUnit['taskid'], TEST_TASKID)
        self.assertEqual(testWorkUnit['last_unit_count'], TEST_LAST_UNIT_COUNT)
        self.assertEqual(testWorkUnit['fileid'], TEST_FILEID)
        self.assertEqual(testWorkUnit['run_lumi'].run, TEST_RUN_NUMBER)
        self.assertCountEqual(testWorkUnit['run_lumi'].lumis, [TEST_LUMI]) if PY3 else self.assertItemsEqual(testWorkUnit['run_lumi'].lumis, [TEST_LUMI])

        # Test the defaults we did not set
        self.assertEqual(testWorkUnit['retry_count'], 0)
        self.assertGreater(testWorkUnit['last_submit_time'], 0)
        self.assertEqual(testWorkUnit['status'], 0)
        self.assertGreaterEqual(testWorkUnit['firstevent'], 0)
        self.assertGreaterEqual(testWorkUnit['lastevent'], 0)

        return

    def testWorkUnitDefinitionNotDefault(self):
        """
        This tests the construction of a DataStructs WorkUnit object with non-default values
        """

        # Things with default values
        testRetries = 10
        testSubmitTime = 10 * 365 * 24 * 3600
        testStatus = 4
        testFirstEvent = 100
        testLastEvent = 600

        testRunLumi = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                runLumi=testRunLumi, retryCount=testRetries, lastSubmitTime=testSubmitTime,
                                status=testStatus, firstEvent=testFirstEvent, lastEvent=testLastEvent)

        # Test the defaults we overrode
        self.assertEqual(testWorkUnit['retry_count'], testRetries)
        self.assertEqual(testWorkUnit['last_submit_time'], testSubmitTime)
        self.assertEqual(testWorkUnit['status'], testStatus)
        self.assertEqual(testWorkUnit['firstevent'], testFirstEvent)
        self.assertEqual(testWorkUnit['lastevent'], testLastEvent)

        return

    def testWorkUnitHashAndCompare(self):
        """
        Test that the hash function works and that the comparisons work
        """

        testRunLumi0 = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit0 = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                 runLumi=testRunLumi0)
        testRunLumi1 = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit1 = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                 runLumi=testRunLumi1)
        testRunLumi2 = Run(TEST_RUN_NUMBER, TEST_LUMI + 1)
        testWorkUnit2 = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                 runLumi=testRunLumi2)
        testRunLumi3 = Run(TEST_RUN_NUMBER + 1, TEST_LUMI)
        testWorkUnit3 = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                 runLumi=testRunLumi3)

        # Tests for hashers
        self.assertEqual(hash(testWorkUnit0), hash(testWorkUnit1))
        self.assertNotEqual(hash(testWorkUnit1), hash(testWorkUnit2))
        self.assertNotEqual(hash(testWorkUnit1), hash(testWorkUnit3))

        # Tests for comparisons
        self.assertEqual(testWorkUnit0, testWorkUnit1)
        self.assertLess(testWorkUnit1, testWorkUnit2)
        self.assertLess(testWorkUnit2, testWorkUnit3)

        return

    def testWorkUnitJsonable(self):
        """
        Test that the object can be turned into JSON
        """

        testRunLumi = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                runLumi=testRunLumi)

        self.assertTrue(json.dumps(testWorkUnit.__to_json__()))

    def testGetInfo(self):
        testRunLumi = Run(TEST_RUN_NUMBER, TEST_LUMI)
        testWorkUnit = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                runLumi=testRunLumi)

        info = testWorkUnit.getInfo()

        # Test the things we set
        self.assertEqual(info[0], TEST_TASKID)
        self.assertEqual(info[2], TEST_LAST_UNIT_COUNT)
        self.assertEqual(info[7], TEST_FILEID)
        self.assertEqual(info[8].run, TEST_RUN_NUMBER)
        self.assertCountEqual(info[8].lumis, [TEST_LUMI]) if PY3 else self.assertItemsEqual(info[8].lumis, [TEST_LUMI])

        # Test the defaults we did not set
        self.assertEqual(info[1], 0)
        self.assertGreater(info[3], 0)
        self.assertEqual(info[4], 0)
        self.assertGreaterEqual(info[5], 0)
        self.assertGreaterEqual(info[6], 0)

        # Run another test by overriding defaults of things with default values
        testRetries = 10
        testSubmitTime = 10 * 365 * 24 * 3600
        testStatus = 4
        testFirstEvent = 100
        testLastEvent = 600

        testWorkUnit = WorkUnit(taskID=TEST_TASKID, lastUnitCount=TEST_LAST_UNIT_COUNT, fileid=TEST_FILEID,
                                runLumi=testRunLumi, retryCount=testRetries, lastSubmitTime=testSubmitTime,
                                status=testStatus, firstEvent=testFirstEvent, lastEvent=testLastEvent)
        info = testWorkUnit.getInfo()

        # Test the defaults we overrode
        self.assertEqual(info[1], testRetries)
        self.assertEqual(info[3], testSubmitTime)
        self.assertEqual(info[4], testStatus)
        self.assertEqual(info[5], testFirstEvent)
        self.assertEqual(info[6], testLastEvent)

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# pylint: disable=W0212
# Access to a protected member (_createSubscriptionsInWMBS)  of a client class
"""
_PromptReco_t_

Unit tests for the new Tier1 PromptReconstruction workflow.
"""
from __future__ import print_function
from future.utils import viewitems

import os
import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.Configuration import ConfigSection
from WMCore.DAOFactory import DAOFactory
from WMCore.Database.CMSCouch import CouchServer
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMSpec.StdSpecs.PromptReco import PromptRecoWorkloadFactory
from WMCore.WorkQueue.WMBSHelper import WMBSHelper
from WMQuality.TestInitCouchApp import TestInitCouchApp


class PromptRecoTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Initialize the database and couch.
        """
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setupCouch("promptreco_t", "ConfigCache")
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        couchServer = CouchServer(os.environ["COUCHURL"])
        self.configDatabase = couchServer.connectDatabase("promptreco_t")
        self.testDir = self.testInit.generateWorkDir()

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        self.listTasksByWorkflow = self.daoFactory(classname="Workflow.LoadFromName")
        self.listFilesets = self.daoFactory(classname="Fileset.List")
        self.listSubsMapping = self.daoFactory(classname="Subscriptions.ListSubsAndFilesetsFromWorkflow")
        self.promptSkim = None
        if PY3:
            self.assertItemsEqual = self.assertCountEqual
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out the database.
        """
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        return

    def setupPromptSkimConfigObject(self):
        """
        _setupPromptSkimConfigObject_
        Creates a custom config object for testing
        of the skim functionality
        """
        self.promptSkim = ConfigSection(name="Tier1Skim")
        self.promptSkim.SkimName = "TestSkim1"
        self.promptSkim.DataTier = "RECO"
        self.promptSkim.TwoFileRead = False
        self.promptSkim.ProcessingVersion = "PromptSkim-v1"
        self.promptSkim.ConfigURL = "http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi/CMSSW/Configuration/DataOps/python/prescaleskimmer.py?revision=1.1"

        # def testPromptReco(self):
        #     """
        #     _testPromptReco_

    #
    #        Create a Prompt Reconstruction workflow
    #        and verify it installs into WMBS correctly.
    #        """
    def testPromptRecoWithSkims(self):
        """
        _testT1PromptRecoWithSkim_

        Create a T1 Prompt Reconstruction workflow with PromptSkims
        and verify it installs into WMBS correctly.
        """
        testArguments = PromptRecoWorkloadFactory.getTestArguments()
        testArguments["CouchURL"] = os.environ["COUCHURL"]
        testArguments["EnableHarvesting"] = True
        factory = PromptRecoWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)
        testWorkload.setSpecUrl("somespec")
        testWorkload.setOwnerDetails("dballest@fnal.gov", "T0")

        testWMBSHelper = WMBSHelper(testWorkload, "Reco", "SomeBlock", cachepath=self.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        recoWorkflow = Workflow(name="TestWorkload",
                                task="/TestWorkload/Reco")
        recoWorkflow.load()
        self.assertEqual(len(recoWorkflow.outputMap), len(testArguments["WriteTiers"]) + 1,
                         "Error: Wrong number of WF outputs in the Reco WF.")

        goldenOutputMods = {"write_RECO": "RECO", "write_ALCARECO": "ALCARECO", "write_AOD": "AOD", "write_DQM": "DQM"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            mergedOutput = recoWorkflow.outputMap[fset][0]["merged_output_fileset"]
            unmergedOutput = recoWorkflow.outputMap[fset][0]["output_fileset"]
            mergedOutput.loadData()
            unmergedOutput.loadData()

            if goldenOutputMod != "write_ALCARECO":
                self.assertEqual(mergedOutput.name,
                                 "/TestWorkload/Reco/RecoMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                                 "Error: Merged output fileset is wrong: %s" % mergedOutput.name)
            self.assertEqual(unmergedOutput.name, "/TestWorkload/Reco/unmerged-%s" % fset,
                             "Error: Unmerged output fileset is wrong: %s" % unmergedOutput.name)

        logArchOutput = recoWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = recoWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name, "/TestWorkload/Reco/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name, "/TestWorkload/Reco/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        alcaSkimWorkflow = Workflow(name="TestWorkload",
                                    task="/TestWorkload/Reco/AlcaSkim")
        alcaSkimWorkflow.load()
        self.assertEqual(len(alcaSkimWorkflow.outputMap), len(testArguments["AlcaSkims"]) + 1,
                         "Error: Wrong number of WF outputs in the AlcaSkim WF.")

        goldenOutputMods = []
        for alcaProd in testArguments["AlcaSkims"]:
            goldenOutputMods.append("ALCARECOStream%s" % alcaProd)

        for goldenOutputMod in goldenOutputMods:
            fset = goldenOutputMod + "ALCARECO"
            mergedOutput = alcaSkimWorkflow.outputMap[fset][0]["merged_output_fileset"]
            unmergedOutput = alcaSkimWorkflow.outputMap[fset][0]["output_fileset"]
            mergedOutput.loadData()
            unmergedOutput.loadData()
            self.assertEqual(mergedOutput.name,
                             "/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-MergedALCARECO" % goldenOutputMod,
                             "Error: Merged output fileset is wrong: %s" % mergedOutput.name)
            self.assertEqual(unmergedOutput.name, "/TestWorkload/Reco/AlcaSkim/unmerged-%sALCARECO" % goldenOutputMod,
                             "Error: Unmerged output fileset is wrong: %s" % unmergedOutput.name)

        logArchOutput = alcaSkimWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = alcaSkimWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name, "/TestWorkload/Reco/AlcaSkim/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name, "/TestWorkload/Reco/AlcaSkim/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        dqmWorkflow = Workflow(name="TestWorkload",
                               task="/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged")
        dqmWorkflow.load()

        logArchOutput = dqmWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = dqmWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name,
                         "/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name,
                         "/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        goldenOutputMods = {"write_RECO": "RECO", "write_AOD": "AOD", "write_DQM": "DQM"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            mergeWorkflow = Workflow(name="TestWorkload",
                                     task="/TestWorkload/Reco/RecoMerge%s" % goldenOutputMod)
            mergeWorkflow.load()

            self.assertEqual(len(mergeWorkflow.outputMap), 2,
                             "Error: Wrong number of WF outputs.")

            mergedMergeOutput = mergeWorkflow.outputMap["Merged%s" % tier][0]["merged_output_fileset"]
            unmergedMergeOutput = mergeWorkflow.outputMap["Merged%s" % tier][0]["output_fileset"]

            mergedMergeOutput.loadData()
            unmergedMergeOutput.loadData()

            self.assertEqual(mergedMergeOutput.name,
                             "/TestWorkload/Reco/RecoMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                             "Error: Merged output fileset is wrong.")
            self.assertEqual(unmergedMergeOutput.name,
                             "/TestWorkload/Reco/RecoMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                             "Error: Unmerged output fileset is wrong.")

            logArchOutput = mergeWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
            unmergedLogArchOutput = mergeWorkflow.outputMap["logArchive"][0]["output_fileset"]
            logArchOutput.loadData()
            unmergedLogArchOutput.loadData()

            self.assertEqual(logArchOutput.name, "/TestWorkload/Reco/RecoMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong: %s" % logArchOutput.name)
            self.assertEqual(unmergedLogArchOutput.name,
                             "/TestWorkload/Reco/RecoMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong.")

        goldenOutputMods = []
        for alcaProd in testArguments["AlcaSkims"]:
            goldenOutputMods.append("ALCARECOStream%s" % alcaProd)

        for goldenOutputMod in goldenOutputMods:
            mergeWorkflow = Workflow(name="TestWorkload",
                                     task="/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s" % goldenOutputMod)
            mergeWorkflow.load()

            self.assertEqual(len(mergeWorkflow.outputMap), 2,
                             "Error: Wrong number of WF outputs %d." % len(mergeWorkflow.outputMap))

            mergedMergeOutput = mergeWorkflow.outputMap["MergedALCARECO"][0]["merged_output_fileset"]
            unmergedMergeOutput = mergeWorkflow.outputMap["MergedALCARECO"][0]["output_fileset"]

            mergedMergeOutput.loadData()
            unmergedMergeOutput.loadData()

            self.assertEqual(mergedMergeOutput.name,
                             "/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-MergedALCARECO" % goldenOutputMod,
                             "Error: Merged output fileset is wrong.")
            self.assertEqual(unmergedMergeOutput.name,
                             "/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-MergedALCARECO" % goldenOutputMod,
                             "Error: Unmerged output fileset is wrong.")

            logArchOutput = mergeWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
            unmergedLogArchOutput = mergeWorkflow.outputMap["logArchive"][0]["output_fileset"]
            logArchOutput.loadData()
            unmergedLogArchOutput.loadData()

            self.assertEqual(logArchOutput.name,
                             "/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong: %s" % logArchOutput.name)
            self.assertEqual(unmergedLogArchOutput.name,
                             "/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong.")

        topLevelFileset = Fileset(name="TestWorkload-Reco-SomeBlock")
        topLevelFileset.loadData()

        recoSubscription = Subscription(fileset=topLevelFileset, workflow=recoWorkflow)
        recoSubscription.loadData()

        self.assertEqual(recoSubscription["type"], "Processing",
                         "Error: Wrong subscription type.")
        self.assertEqual(recoSubscription["split_algo"], "EventAwareLumiBased",
                         "Error: Wrong split algorithm. %s" % recoSubscription["split_algo"])

        alcaRecoFileset = Fileset(name="/TestWorkload/Reco/unmerged-write_ALCARECOALCARECO")
        alcaRecoFileset.loadData()

        alcaSkimSubscription = Subscription(fileset=alcaRecoFileset, workflow=alcaSkimWorkflow)
        alcaSkimSubscription.loadData()

        self.assertEqual(alcaSkimSubscription["type"], "Processing",
                         "Error: Wrong subscription type.")
        self.assertEqual(alcaSkimSubscription["split_algo"], "ParentlessMergeBySize",
                         "Error: Wrong split algorithm. %s" % alcaSkimSubscription["split_algo"])

        mergedDQMFileset = Fileset(name="/TestWorkload/Reco/RecoMergewrite_DQM/merged-MergedDQM")
        mergedDQMFileset.loadData()

        dqmSubscription = Subscription(fileset=mergedDQMFileset, workflow=dqmWorkflow)
        dqmSubscription.loadData()

        self.assertEqual(dqmSubscription["type"], "Harvesting",
                         "Error: Wrong subscription type.")
        self.assertEqual(dqmSubscription["split_algo"], "Harvest",
                         "Error: Wrong split algo.")

        unmergedOutputs = {"write_RECO": "RECO", "write_AOD": "AOD", "write_DQM": "DQM"}
        for unmergedOutput, tier in viewitems(unmergedOutputs):
            fset = unmergedOutput + tier
            unmergedDataTier = Fileset(name="/TestWorkload/Reco/unmerged-%s" % fset)
            unmergedDataTier.loadData()
            dataTierMergeWorkflow = Workflow(name="TestWorkload",
                                             task="/TestWorkload/Reco/RecoMerge%s" % unmergedOutput)
            dataTierMergeWorkflow.load()
            mergeSubscription = Subscription(fileset=unmergedDataTier, workflow=dataTierMergeWorkflow)
            mergeSubscription.loadData()

            self.assertEqual(mergeSubscription["type"], "Merge",
                             "Error: Wrong subscription type.")
            self.assertEqual(mergeSubscription["split_algo"], "ParentlessMergeBySize",
                             "Error: Wrong split algorithm. %s" % mergeSubscription["split_algo"])
        unmergedOutputs = []
        for alcaProd in testArguments["AlcaSkims"]:
            unmergedOutputs.append("ALCARECOStream%s" % alcaProd)
        for unmergedOutput in unmergedOutputs:
            unmergedAlcaSkim = Fileset(name="/TestWorkload/Reco/AlcaSkim/unmerged-%sALCARECO" % unmergedOutput)
            unmergedAlcaSkim.loadData()
            alcaSkimMergeWorkflow = Workflow(name="TestWorkload",
                                             task="/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s" % unmergedOutput)
            alcaSkimMergeWorkflow.load()
            mergeSubscription = Subscription(fileset=unmergedAlcaSkim, workflow=alcaSkimMergeWorkflow)
            mergeSubscription.loadData()

            self.assertEqual(mergeSubscription["type"], "Merge",
                             "Error: Wrong subscription type.")
            self.assertEqual(mergeSubscription["split_algo"], "ParentlessMergeBySize",
                             "Error: Wrong split algorithm. %s" % mergeSubscription["split_algo"])

        goldenOutputMods = {"write_RECO": "RECO", "write_AOD": "AOD", "write_DQM": "DQM"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            unmergedFileset = Fileset(name="/TestWorkload/Reco/unmerged-%s" % fset)
            unmergedFileset.loadData()
            cleanupWorkflow = Workflow(name="TestWorkload",
                                       task="/TestWorkload/Reco/RecoCleanupUnmerged%s" % goldenOutputMod)
            cleanupWorkflow.load()
            cleanupSubscription = Subscription(fileset=unmergedFileset, workflow=cleanupWorkflow)
            cleanupSubscription.loadData()

            self.assertEqual(cleanupSubscription["type"], "Cleanup",
                             "Error: Wrong subscription type.")
            self.assertEqual(cleanupSubscription["split_algo"], "SiblingProcessingBased",
                             "Error: Wrong subscription type.")

        goldenOutputMods = []
        for alcaProd in testArguments["AlcaSkims"]:
            goldenOutputMods.append("ALCARECOStream%s" % alcaProd)
        for goldenOutputMod in goldenOutputMods:
            unmergedFileset = Fileset(name="/TestWorkload/Reco/AlcaSkim/unmerged-%sALCARECO" % goldenOutputMod)
            unmergedFileset.loadData()
            cleanupWorkflow = Workflow(name="TestWorkload",
                                       task="/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmerged%s" % goldenOutputMod)
            cleanupWorkflow.load()
            cleanupSubscription = Subscription(fileset=unmergedFileset, workflow=cleanupWorkflow)
            cleanupSubscription.loadData()

            self.assertEqual(cleanupSubscription["type"], "Cleanup",
                             "Error: Wrong subscription type.")
            self.assertEqual(cleanupSubscription["split_algo"], "SiblingProcessingBased",
                             "Error: Wrong subscription type.")

        recoLogCollect = Fileset(name="/TestWorkload/Reco/unmerged-logArchive")
        recoLogCollect.loadData()
        recoLogCollectWorkflow = Workflow(name="TestWorkload",
                                          task="/TestWorkload/Reco/LogCollect")
        recoLogCollectWorkflow.load()
        logCollectSub = Subscription(fileset=recoLogCollect, workflow=recoLogCollectWorkflow)
        logCollectSub.loadData()

        self.assertEqual(logCollectSub["type"], "LogCollect",
                         "Error: Wrong subscription type.")
        self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                         "Error: Wrong split algorithm.")

        alcaSkimLogCollect = Fileset(name="/TestWorkload/Reco/AlcaSkim/unmerged-logArchive")
        alcaSkimLogCollect.loadData()
        alcaSkimLogCollectWorkflow = Workflow(name="TestWorkload",
                                              task="/TestWorkload/Reco/AlcaSkim/AlcaSkimLogCollect")
        alcaSkimLogCollectWorkflow.load()
        logCollectSub = Subscription(fileset=alcaSkimLogCollect, workflow=alcaSkimLogCollectWorkflow)
        logCollectSub.loadData()

        self.assertEqual(logCollectSub["type"], "LogCollect",
                         "Error: Wrong subscription type.")
        self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                         "Error: Wrong split algorithm.")

        goldenOutputMods = ["write_RECO", "write_AOD", "write_DQM"]
        for goldenOutputMod in goldenOutputMods:
            recoMergeLogCollect = Fileset(name="/TestWorkload/Reco/RecoMerge%s/merged-logArchive" % goldenOutputMod)
            recoMergeLogCollect.loadData()
            recoMergeLogCollectWorkflow = Workflow(name="TestWorkload",
                                                   task="/TestWorkload/Reco/RecoMerge%s/Reco%sMergeLogCollect" % (
                                                       goldenOutputMod, goldenOutputMod))
            recoMergeLogCollectWorkflow.load()
            logCollectSubscription = Subscription(fileset=recoMergeLogCollect, workflow=recoMergeLogCollectWorkflow)
            logCollectSubscription.loadData()

            self.assertEqual(logCollectSub["type"], "LogCollect",
                             "Error: Wrong subscription type.")
            self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                             "Error: Wrong split algorithm.")

        goldenOutputMods = []
        for alcaProd in testArguments["AlcaSkims"]:
            goldenOutputMods.append("ALCARECOStream%s" % alcaProd)
        for goldenOutputMod in goldenOutputMods:
            alcaSkimLogCollect = Fileset(
                name="/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/merged-logArchive" % goldenOutputMod)
            alcaSkimLogCollect.loadData()
            alcaSkimLogCollectWorkflow = Workflow(name="TestWorkload",
                                                  task="/TestWorkload/Reco/AlcaSkim/AlcaSkimMerge%s/AlcaSkim%sMergeLogCollect" % (
                                                      goldenOutputMod, goldenOutputMod))
            alcaSkimLogCollectWorkflow.load()
            logCollectSubscription = Subscription(fileset=alcaSkimLogCollect, workflow=alcaSkimLogCollectWorkflow)
            logCollectSubscription.loadData()

            self.assertEqual(logCollectSub["type"], "LogCollect",
                             "Error: Wrong subscription type.")
            self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                             "Error: Wrong split algorithm.")

        dqmHarvestLogCollect = Fileset(
            name="/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/unmerged-logArchive")
        dqmHarvestLogCollect.loadData()
        dqmHarvestLogCollectWorkflow = Workflow(name="TestWorkload",
                                                task="/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/RecoMergewrite_DQMMergedEndOfRunDQMHarvestLogCollect")
        dqmHarvestLogCollectWorkflow.load()

        logCollectSub = Subscription(fileset=dqmHarvestLogCollect, workflow=dqmHarvestLogCollectWorkflow)
        logCollectSub.loadData()

        self.assertEqual(logCollectSub["type"], "LogCollect",
                         "Error: Wrong subscription type.")
        self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                         "Error: Wrong split algo.")

        return

    def testMemCoresSettings(self):
        """
        _testMemCoresSettings_

        Make sure the multicore and memory setings are properly propagated to
        all tasks and steps.
        """
        testArguments = PromptRecoWorkloadFactory.getTestArguments()
        testArguments["CouchURL"] = os.environ["COUCHURL"]
        testArguments["CouchDBName"] = "promptreco_t"
        testArguments["EnableHarvesting"] = True

        factory = PromptRecoWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        # test default values
        taskPaths = ['/TestWorkload/Reco', '/TestWorkload/Reco/AlcaSkim']
        for task in taskPaths:
            taskObj = testWorkload.getTaskByPath(task)
            for step in ('cmsRun1', 'stageOut1', 'logArch1'):
                stepHelper = taskObj.getStepHelper(step)
                self.assertEqual(stepHelper.getNumberOfCores(), 1)
                self.assertEqual(stepHelper.getNumberOfStreams(), 0)
            # then test Memory requirements
            perfParams = taskObj.jobSplittingParameters()['performance']
            self.assertEqual(perfParams['memoryRequirement'], 2300.0)

        # now test case where args are provided
        testArguments["Multicore"] = 6
        testArguments["Memory"] = 4600.0
        testArguments["EventStreams"] = 3
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)
        for task in taskPaths:
            taskObj = testWorkload.getTaskByPath(task)
            for step in ('cmsRun1', 'stageOut1', 'logArch1'):
                stepHelper = taskObj.getStepHelper(step)
                if task == '/TestWorkload/Reco' and step == 'cmsRun1':
                    self.assertEqual(stepHelper.getNumberOfCores(), testArguments["Multicore"])
                    self.assertEqual(stepHelper.getNumberOfStreams(), testArguments["EventStreams"])
                elif step in ('stageOut1', 'logArch1'):
                    self.assertEqual(stepHelper.getNumberOfCores(), 1)
                    self.assertEqual(stepHelper.getNumberOfStreams(), 0)
                else:
                    self.assertEqual(stepHelper.getNumberOfCores(), 1, "%s should be single-core" % task)
                    self.assertEqual(stepHelper.getNumberOfStreams(), 0)
            # then test Memory requirements
            perfParams = taskObj.jobSplittingParameters()['performance']
            self.assertEqual(perfParams['memoryRequirement'], testArguments["Memory"])

        return

    def testFilesets(self):
        """
        Test workflow tasks, filesets and subscriptions creation
        """
        # expected tasks, filesets, subscriptions, etc
        expOutTasks = ['/TestWorkload/Reco',
                       '/TestWorkload/Reco/AlcaSkim',
                       '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics',
                       '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T',
                       '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics',
                       '/TestWorkload/Reco/RecoMergewrite_AOD',
                       '/TestWorkload/Reco/RecoMergewrite_DQM',
                       '/TestWorkload/Reco/RecoMergewrite_RECO']
        expWfTasks = ['/TestWorkload/Reco',
                      '/TestWorkload/Reco/AlcaSkim',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamHcalCalHOCosmics',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamMuAlGlobalCosmics',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamTkAlCosmics0T',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimLogCollect',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics/AlcaSkimALCARECOStreamHcalCalHOCosmicsMergeLogCollect',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics/AlcaSkimALCARECOStreamMuAlGlobalCosmicsMergeLogCollect',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T',
                      '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T/AlcaSkimALCARECOStreamTkAlCosmics0TMergeLogCollect',
                      '/TestWorkload/Reco/LogCollect',
                      '/TestWorkload/Reco/RecoCleanupUnmergedwrite_ALCARECO',
                      '/TestWorkload/Reco/RecoCleanupUnmergedwrite_AOD',
                      '/TestWorkload/Reco/RecoCleanupUnmergedwrite_DQM',
                      '/TestWorkload/Reco/RecoCleanupUnmergedwrite_RECO',
                      '/TestWorkload/Reco/RecoMergewrite_AOD',
                      '/TestWorkload/Reco/RecoMergewrite_AOD/Recowrite_AODMergeLogCollect',
                      '/TestWorkload/Reco/RecoMergewrite_DQM',
                      '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged',
                      '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/RecoMergewrite_DQMMergedEndOfRunDQMHarvestLogCollect',
                      '/TestWorkload/Reco/RecoMergewrite_DQM/Recowrite_DQMMergeLogCollect',
                      '/TestWorkload/Reco/RecoMergewrite_RECO',
                      '/TestWorkload/Reco/RecoMergewrite_RECO/Recowrite_RECOMergeLogCollect']
        expFsets = ['TestWorkload-Reco-/MinimumBias/ComissioningHI-v1/RAW',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics/merged-MergedALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamHcalCalHOCosmicsALCARECO',
                    '/TestWorkload/Reco/unmerged-write_ALCARECOALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics/merged-MergedALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T/merged-MergedALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamMuAlGlobalCosmicsALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamTkAlCosmics0TALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/unmerged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_AOD/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_AOD/merged-MergedAOD',
                    '/TestWorkload/Reco/unmerged-write_AODAOD',
                    '/TestWorkload/Reco/unmerged-write_DQMDQM',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/merged-MergedDQM',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_RECO/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_RECO/merged-MergedRECO',
                    '/TestWorkload/Reco/unmerged-logArchive',
                    '/TestWorkload/Reco/unmerged-write_RECORECO']
        subMaps = [(5,
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics/AlcaSkimALCARECOStreamHcalCalHOCosmicsMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (8,
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics/AlcaSkimALCARECOStreamMuAlGlobalCosmicsMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (11,
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T/merged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T/AlcaSkimALCARECOStreamTkAlCosmics0TMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (3,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamHcalCalHOCosmicsALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamHcalCalHOCosmics',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (4,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamHcalCalHOCosmicsALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (6,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamMuAlGlobalCosmicsALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamMuAlGlobalCosmics',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (7,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamMuAlGlobalCosmicsALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (9,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamTkAlCosmics0TALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimCleanupUnmergedALCARECOStreamTkAlCosmics0T',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (10,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-ALCARECOStreamTkAlCosmics0TALCARECO',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (12,
                    '/TestWorkload/Reco/AlcaSkim/unmerged-logArchive',
                    '/TestWorkload/Reco/AlcaSkim/AlcaSkimLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (16,
                    '/TestWorkload/Reco/RecoMergewrite_AOD/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_AOD/Recowrite_AODMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (21,
                    '/TestWorkload/Reco/RecoMergewrite_DQM/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/Recowrite_DQMMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (19,
                    '/TestWorkload/Reco/RecoMergewrite_DQM/merged-MergedDQM',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged',
                    'Harvest',
                    'Harvesting'),
                   (20,
                    '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged/RecoMergewrite_DQMMergedEndOfRunDQMHarvestLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (24,
                    '/TestWorkload/Reco/RecoMergewrite_RECO/merged-logArchive',
                    '/TestWorkload/Reco/RecoMergewrite_RECO/Recowrite_RECOMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (25,
                    '/TestWorkload/Reco/unmerged-logArchive',
                    '/TestWorkload/Reco/LogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (2,
                    '/TestWorkload/Reco/unmerged-write_ALCARECOALCARECO',
                    '/TestWorkload/Reco/AlcaSkim',
                    'ParentlessMergeBySize',
                    'Processing'),
                   (13,
                    '/TestWorkload/Reco/unmerged-write_ALCARECOALCARECO',
                    '/TestWorkload/Reco/RecoCleanupUnmergedwrite_ALCARECO',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (14,
                    '/TestWorkload/Reco/unmerged-write_AODAOD',
                    '/TestWorkload/Reco/RecoCleanupUnmergedwrite_AOD',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (15,
                    '/TestWorkload/Reco/unmerged-write_AODAOD',
                    '/TestWorkload/Reco/RecoMergewrite_AOD',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (17,
                    '/TestWorkload/Reco/unmerged-write_DQMDQM',
                    '/TestWorkload/Reco/RecoCleanupUnmergedwrite_DQM',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (18,
                    '/TestWorkload/Reco/unmerged-write_DQMDQM',
                    '/TestWorkload/Reco/RecoMergewrite_DQM',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (22,
                    '/TestWorkload/Reco/unmerged-write_RECORECO',
                    '/TestWorkload/Reco/RecoCleanupUnmergedwrite_RECO',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (23,
                    '/TestWorkload/Reco/unmerged-write_RECORECO',
                    '/TestWorkload/Reco/RecoMergewrite_RECO',
                    'ParentlessMergeBySize',
                    'Merge'),
                   (1,
                    'TestWorkload-Reco-/MinimumBias/ComissioningHI-v1/RAW',
                    '/TestWorkload/Reco',
                    'EventAwareLumiBased',
                    'Processing')]

        testArguments = PromptRecoWorkloadFactory.getTestArguments()
        testArguments["CouchURL"] = os.environ["COUCHURL"]
        testArguments["CouchDBName"] = "promptreco_t"
        testArguments["EnableHarvesting"] = True

        factory = PromptRecoWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        testWMBSHelper = WMBSHelper(testWorkload, "Reco", blockName=testArguments['InputDataset'],
                                    cachepath=self.testInit.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        self.assertItemsEqual(testWorkload.listOutputProducingTasks(), expOutTasks)

        workflows = self.listTasksByWorkflow.execute(workflow="TestWorkload")
        self.assertItemsEqual([item['task'] for item in workflows], expWfTasks)

        # returns a tuple of id, name, open and last_update
        filesets = self.listFilesets.execute()
        self.assertItemsEqual([item[1] for item in filesets], expFsets)

        subscriptions = self.listSubsMapping.execute(workflow="TestWorkload", returnTuple=True)
        self.assertItemsEqual(subscriptions, subMaps)


    def testallowCreationFailureArgExists(self):
        """
        Test allowCreationFailure arguments exists.
        """
        testArguments = PromptRecoWorkloadFactory.getTestArguments()
        testArguments["CouchURL"] = os.environ["COUCHURL"]
        testArguments["EnableHarvesting"] = True
        factory = PromptRecoWorkloadFactory()
        factory.factoryWorkloadConstruction("TestWorkload", testArguments)
        factory.procJobSplitArgs["allowCreationFailure"] = False
        self.assertItemsEqual(factory.procJobSplitArgs, {'events_per_job': 500, 'allowCreationFailure': False, 'job_time_limit': 345600})

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Exists_

Oracle implementation of Fileset.Exists

"""
__all__ = []



from WMCore.WMBS.MySQL.Fileset.Exists import Exists as ExistsFilesetMySQL

class Exists(ExistsFilesetMySQL):
    sql = ExistsFilesetMySQL.sql



#!/usr/bin/env python
"""
_Dummy_

A dummy class to mimic the component for tets.
"""




import logging

class Dummy:
    """
    _Dummy_

    A dummy class to mimic the component for tets.
    """

    def __init__(self):
        logging.debug("I am a dummy object!")
        self.config = None



#!/usr/bin/env python
"""
    WorkQueue tests
"""
from __future__ import absolute_import, division, print_function

import cProfile
import pstats
import shutil
import tempfile
import unittest

from WMCore_t.WorkQueue_t.WorkQueueTestCase import WorkQueueTestCase

from WMCore.WorkQueue.WorkQueue import localQueue
from WMQuality.Emulators.RucioClient.MockRucioApi import SITES as DUMMY_SITES
from WMQuality.Emulators.WMSpecGenerator.WMSpecGenerator import WMSpecGenerator


class LocalWorkQueueProfileTest(WorkQueueTestCase):
    """
    _WorkQueueTest_

    """

    def setUp(self):
        """
        If we dont have a wmspec file create one
        """

        WorkQueueTestCase.setUp(self)

        self.cacheDir = tempfile.mkdtemp()
        self.specGenerator = WMSpecGenerator(self.cacheDir)
        self.specs = self.createReRecoSpec(1, "file")
        # setup rucio parameters for global/local queue
        self.queueParams = {}
        self.queueParams['log_reporter'] = "lq_profile_test"
        self.queueParams['rucioAccount'] = "wma_test"
        self.queueParams['rucioAuthUrl'] = "http://cms-rucio-int.cern.ch"
        self.queueParams['rucioUrl'] = "https://cms-rucio-auth-int.cern.ch"

        # Create queues
        self.localQueue = localQueue(DbName=self.queueDB, InboxDbName=self.queueInboxDB,
                                     NegotiationTimeout=0, QueueURL='global.example.com', CacheDir=self.cacheDir,
                                     central_logdb_url="%s/%s" % (self.couchURL, self.logDBName),
                                     **self.queueParams)


    def tearDown(self):
        """tearDown"""
        WorkQueueTestCase.tearDown(self)
        try:
            shutil.rmtree(self.cacheDir)
            self.specGenerator.removeSpecs()
        except Exception:
            pass

    def createReRecoSpec(self, numOfSpec, kind="spec"):
        specs = []
        for i in range(numOfSpec):
            specName = "MinBiasProcessingSpec_Test_%s" % (i+1)
            specs.append(self.specGenerator.createReRecoSpec(specName, kind,
                                                             assignKwargs={'SiteWhitelist': ['T2_XX_SiteA']}))
        return specs

    def createProfile(self, name, function):
        fileName = name
        prof = cProfile.Profile()
        prof.runcall(function)
        prof.dump_stats(fileName)
        p = pstats.Stats(fileName)
        p.strip_dirs().sort_stats('cumulative').print_stats(0.1)
        p.strip_dirs().sort_stats('time').print_stats(0.1)
        p.strip_dirs().sort_stats('calls').print_stats(0.1)
        p.strip_dirs().sort_stats('name').print_stats(10)

    def testGetWorkLocalQueue(self):
        i = 0
        for spec in self.specs:
            i += 1
            specName = "MinBiasProcessingSpec_Test_%s" % i
            self.localQueue.queueWork(spec, specName, team="A-team")
        self.localQueue.updateLocationInfo()
        self.createProfile('getWorkProfile.prof', self.localQueueGetWork)

    def localQueueGetWork(self):
        siteJobs = {}
        for site in DUMMY_SITES:
            siteJobs[site] = 100000
        self.localQueue.getWork(siteJobs, {})


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
brokenCmsRun.py

cmsRun simulator that exits with a non zero exit code.

Created on Feb 26, 2013

@author: dballest
"""

import sys

if __name__ == '__main__':
    sys.exit(65)



#!/usr/bin/env python

"""
_TestComponent_

Compnent to test the skeleton and serve as an example to build
components.


"""





import logging

# harness class that encapsulates the basic component logic.
from WMCore.Agent.Harness import Harness

# just the base handler for some tests.
from WMCore.Agent.BaseHandler import BaseHandler

class TestComponent(Harness):
    """
    _TestComponent_

    Compnent to test the skeleton and serve as an example to build
    components.

    """

    def __init__(self, config):
        # call the base class
        Harness.__init__(self, config)

    def logState(self):
        """
        Augment standard logging message.
        """

        state = 'This is the state of the TestComponent\n'
        state += '-------------------------------------'
        return state

    def preInitialization(self):
        """
        Override pre initialization.
        """
        aHandlerInstance = BaseHandler(self)
        # obviously this does not have to be a 1 on 1 mapping.
        # we can have one instance for several messages
        self.messages['TestMessage1'] = aHandlerInstance
        self.messages['TestMessage2'] = aHandlerInstance
        # or multiple instances for different messages.
        self.messages['TestMessage3'] = BaseHandler(self)
        self.messages['TestMessage4'] = BaseHandler(self)

        logging.debug("TestComponent pre initialization")

    def postInitialization(self):
        """
        Override post initialization.
        """

        logging.debug("TestComponent post initialization")



from builtins import range
import random
import time

NUM_OF_REQUEST = 30
ITERATIONS = 50  # number of jobs updated
NUM_OF_JOBS_PER_REQUEST = 10


# To use curl to insert doc
# curl -d @sample_docs.json -X POST -H "Content-Type:application/json" $DB_URL/_bulk_docs

def generate_reqmgr_requests(number=NUM_OF_REQUEST):
    """
    generate the request with following structure
      {
       "_id": "cmsdataops_sryu_test4_120111_114950_128",
       "_rev": "6-02b17b4eabcf333e7499c0fa0ae5055b",
       "inputdataset": "/Photon/Run2011A-v1/RAW",
       "group": "cmsdataops",
       "request_date": [2012, 1, 11, 17, 49, 50],
       "campaign": "SryuTest2",
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "priority": "1",
       "requestor": "cmsdataops",
       "request_type": "ReReco",
       "type": "reqmgr_request",
       "request_status": [
           {
               "status": "new",
               "update_time": 1326304190
           },
           {
               "status": "assignment-approved",
               "update_time": 1326304216
           },
           {
               "status": "assigned",
               "update_time": 1326304227
           }
       ],
       "site_white_list": [
           "T1_DE_KIT"
       ],
       "team": "cmsdataops",
    }
    """
    docs = []
    for i in range(number):
        doc = {"_id": "test_workflow_%s" % i,
               "inputdataset": "/Photon/Run2011A-v1/RAW",
               "group": "cmsdataops",
               "request_date": [2012, 1, 11, 17, 49, 50],
               "campaign": "SryuTest-%s" % (i % 5),
               "workflow": "test_workflow_%s" % i,
               "priority": "1",
               "requestor": "cmsdataops",
               "request_type": "ReReco",
               "type": "reqmgr_request",
               "request_status": [
                   {"status": "new", "update_time": 1326304190},
                   {"status": "assignment-approved", "update_time": 1326304216},
                   {"status": "assigned", "update_time": 1326304227}
               ],
               "site_white_list": ["T1_DE_KIT"],
               "team": "cmsdataops"
               }
        docs.append(doc)
    return docs


def generate_agent_requests(number=NUM_OF_REQUEST, iterations=ITERATIONS):
    """
    generate the request with following structure
      {
       "_id": "af27057919546ff8f3fc8d7f18233355",
       "_rev": "1-181021c38a5444676d7718f42ffa9a89",
       "status": {
           "inWMBS": 1,
           "queued": {
               "retry": 1
           }
       },
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "timestamp": 1326306397,
       "sites": {
           "T1_DE_KIT": {
               "queued": {
                   "retry": 1
               }
           }
       },
       "agent": "WMAgent",
       "team": "team1,team2,cmsdataops",
       "agent_url": "cms-xen39.fnal.gov",
       "type": "agent_request"
       }
    """
    current_time = int(time.time())
    docs = []
    for cycle in range(iterations):
        for i in range(number):
            doc = {"status": {"inWMBS": 12,
                              "submitted": {"retry": 2, "running": 2, "pending": 2, "first": 2},
                              "failure": {"exception": 2, "create": 2, "submit": 2},
                              "queued": {"retry": 2, "first": 2},
                              "canceled": 2,
                              "cooloff": 2,
                              "success": 2
                              },

                   "workflow": "test_workflow_%s" % i,
                   "timestamp": current_time + (cycle * 10),
                   "sites": {"T1_DE_KIT":
                       {
                           "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                           "failure": {"exception": 1, "create": 1, "submit": 1},
                           "queued": {"retry": 1, "first": 1},
                           "canceled": 1,
                           "cooloff": 1,
                           "success": 1
                       },
                       "T1_US_FNAL":
                           {
                               "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                               "failure": {"exception": 1, "create": 1, "submit": 1},
                               "queued": {"retry": 1, "first": 1},
                               "canceled": 1,
                               "cooloff": 1,
                               "success": 1
                           }
                   },
                   "agent": "WMAgent",
                   "agent_teams": "cmsdataops",
                   "agent_url": "cms-xen39.fnal.gov",
                   "type": "agent_request"
                   }
            docs.append(doc)
    return docs


def generate_jobsummary(request, number=NUM_OF_JOBS_PER_REQUEST):
    """
    jobSummary = {"_id": "jobid_1",  //jobid
                  "type": "jobsummary", // setvalue
                  "retrycount": job["retry_count"],
                  "workflow": workflow1, //request name
                  "task": job["task"],
                  "state": success,
                  "site": T1_US_FNAL,
                  "exitcode": 123,
                  "errors": errmsgs,
                  "lumis": inputs,
                  "output": outputs }

    errmsgs = {}
    inputs = []
    for step in fwjrDocument["fwjr"]["steps"]:
        if "errors" in fwjrDocument["fwjr"]["steps"][step]:
            errmsgs[step] = [error for error in fwjrDocument["fwjr"]["steps"][step]["errors"]]
        if "input" in fwjrDocument["fwjr"]["steps"][step] and "source" in fwjrDocument["fwjr"]["steps"][step]["input"]:
            inputs.extend( [source["runs"] for source in fwjrDocument["fwjr"]['steps'][step]["input"]["source"] if "runs" in source] )
    outputs = [ {'type': singlefile.get('module_label', None),
                 'lfn': singlefile.get('lfn', None),
                 'location': singlefile.get('locations', None),
                 'checksums': singlefile.get('checksums', {}),
                     'size': singlefile.get('size', None) } for singlefile in job["fwjr"].getAllFiles() if singlefile ]


    job status
    ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
     'jobfailed', 'createcooloff',  'submitcooloff', 'jobcooloff', 'success',
     'exhausted', 'killed']
    """

    # TODO: Make more realistic
    docs = []
    statusList = ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
                  'jobfailed', 'createcooloff', 'submitcooloff', 'jobcooloff', 'success',
                  'exhausted', 'killed']

    for i in range(number):
        status = statusList[random.randint(0, len(statusList) - 1)]
        errmsgs = {}
        if status.find("failed"):
            exitCode = 666
            errmsgs["step1"] = {}
            errmsgs["step1"]["out"] = {}
            errmsgs["step1"]["out"]["type"] = "test error"
        else:
            exitCode = 0

        jobSummary = {"_id": "jobid_%s_%s" % (request, i),
                      "type": "jobsummary",
                      "retrycount": random.randint(0, 5),
                      "workflow": request,
                      "task": "/%s/task_%s" % (request, i),
                      "state": status,
                      "site": "T1_US_FNAL",
                      "exitcode": exitCode,
                      "errors": errmsgs,
                      "lumis": [[123, 124], [567, 879]],
                      "output": [{'type': "test-type",
                                  'lfn': "/somewhere/file.root",
                                  'location': ['T1_US_FNAL'],
                                  'checksums': {'adler32': 'abc123', 'cksum': 'cdf123'},
                                  'size': "1000"}]
                      }
        docs.append(jobSummary)
    return docs


if __name__ == "__main__":
    from argparse import ArgumentParser
    import json


    def parse_opts():
        parser = ArgumentParser()
        parser.add_argument("-i", "--iterations",
                            dest="iterations",
                            default=ITERATIONS,
                            type=int,
                            help="The number of iterations to make, default=%s" % ITERATIONS)
        parser.add_argument("-r", "--requests",
                            dest="requests",
                            default=NUM_OF_REQUEST,
                            type=int,
                            help="The number of requests to simulate, default=%s" % NUM_OF_REQUEST)

        return parser.parse_args()


    options = parse_opts()
    reqmgr_requests = generate_reqmgr_requests(options.requests)
    agent_requests = generate_agent_requests(options.requests, options.iterations)

    docList = []
    docList.extend(reqmgr_requests)
    docList.extend(agent_requests)
    for req in agent_requests:
        docList.extend(generate_jobsummary(req['workflow']))
    docs = {"docs": docList}

    json.dump(docs, open("sample_docs.json", "w+"))



#!/usr/bin/env python
"""
A sane default welcome page
"""

import time
from cherrypy import expose
from cherrypy import __version__ as cherrypy_version
from cherrypy.lib import cpstats
from Cheetah import Version as cheetah_version
from WMCore.WebTools.Page import Page

class Welcome(Page):
    """
    A sane default welcome page
    """
    def __init__(self, namesAndDocstrings):
        Page.__init__(self, {})
        self.namesAndDocstrings = namesAndDocstrings

    @expose
    def index(self):
        """
        Return an index page of all the pages in the web app.
        """
        html = '<html>'
        html += '<head>'
        html += '<link rel="stylesheet" type="text/css" '
        html += 'href="/controllers/css/reset/style.css"/>'
        html += '</head>'
        html += '<body>'
        html += '<h1>Welcome to the DMWM web framework</h1>'
        html += '<hr style="width:100%;border-top: 1px dotted #CCCCCC;" />'
        html += "<table>\n"
        html += '<tr><th align="left"><h3>Service</h3></th>\n'
        html += '<th align="left"><h3>Description</h3></th></tr>\n'
        self.namesAndDocstrings.sort()
        for name, docstring in self.namesAndDocstrings:
            html += '<tr><td><p><a href="%s">%s</a></p></td>\n' \
                % (name, name)
            html += '<td><p>%s</p></td></tr>\n' % docstring
        html += '</table><br />'
        html += '</body>'
        html += '</html>'
        return html

    @expose
    def default(self, *args, **kwargs):
        """
        Return the index.
        """
        return self.index()

    @expose
    def stats(self):
        "Return CherryPy stats dict about underlying service activities"
        return cpstats.StatsPage().data()



#!/usr/bin/env python
"""
_GFAL2Impl_
Implementation of StageOutImpl interface for gfal-copy
"""
import argparse
import os

from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutImpl import StageOutImpl

_CheckExitCodeOption = True


class GFAL2Impl(StageOutImpl):
    """
    _GFAL2Impl_
    Implement interface for GFAL2 commands (gfal-copy, gfal-rm)
    """

    def __init__(self, stagein=False):
        StageOutImpl.__init__(self, stagein)
        # If we want to execute commands in clean shell, we can`t separate them with ';'.
        # Next commands after separation are executed without env -i and this leads us with
        # mixed environment with COMP and system python.
        # GFAL2 is not build under COMP environment and it had failures with mixed environment.
        self.setups = "env -i X509_USER_PROXY=$X509_USER_PROXY JOBSTARTDIR=$JOBSTARTDIR bash -c '{}'"
        self.removeCommand = self.setups.format('. $JOBSTARTDIR/startup_environment.sh; date; gfal-rm -t 600 {}')
        self.copyOpts = '-t 2400 -T 2400 -p -v --abort-on-failure {checksum} {options} {source} {destination}'
        self.copyCommand = self.setups.format('. $JOBSTARTDIR/startup_environment.sh; date; gfal-copy ' + self.copyOpts)

    def createFinalPFN(self, pfn):
        """
        _createFinalPFN_
        GFAL2 requires file:/// for any direction transfers
        """
        if pfn.startswith('file:'):
            return pfn
        elif os.path.isfile(pfn):
            return "file://{}".format(os.path.abspath(pfn))
        elif pfn.startswith('/'):
            return "file://{}".format(os.path.abspath(pfn))
        return pfn

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_
        GFAL2 uses file:/// urls
        """
        return self.createFinalPFN(pfn)

    def createTargetName(self, protocol, pfn):
        """
        _createTargetName_
        GFAL2 uses file:// urls
        """
        return self.createFinalPFN(pfn)

    def createOutputDirectory(self, targetPFN):
        """
        _createOutputDirectory_
        gfal-copy handles this with option -p and
        it is not needed to create directory
        """
        return

    def createRemoveFileCommand(self, pfn):
        """
        _createRemoveFileCommand_
        handle file remove using gfal-rm

        gfal-rm options used:
          -t   global timeout for the execution of the command.
               Command is interrupted if time expires before it finishes
        """
        if os.path.isfile(pfn):
            return "/bin/rm -f {}".format(os.path.abspath(pfn))
        else:
            return self.removeCommand.format(self.createFinalPFN(pfn))


    def buildCopyCommandDict(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        Build the gfal-cp command for stageOut

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for gfal-cp
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """

        copyCommandDict = {'checksum': '', 'options': '', 'source': '', 'destination': ''}

        useChecksum = (checksums is not None and 'adler32' in checksums and not self.stageIn)

        if not options:
            options = ''

        parser = argparse.ArgumentParser()
        parser.add_argument('--nochecksum', action='store_true')
        args, unknown = parser.parse_known_args(options.split())

        if not args.nochecksum:
            if useChecksum:
                checksums['adler32'] = "{:08x}".format(int(checksums['adler32'], 16))
                copyCommandDict['checksum'] = "-K adler32:{}".format(checksums['adler32'])
            else:
                copyCommandDict['checksum'] = "-K adler32"

        copyCommandDict['options'] = ' '.join(unknown)
        copyCommandDict['source'] = self.createFinalPFN(sourcePFN)
        copyCommandDict['destination'] = self.createFinalPFN(targetPFN)

        return copyCommandDict

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        Create gfal-cp command for stageOut

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for gfal-cp
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """

        copyCommandDict = self.buildCopyCommandDict(sourcePFN, targetPFN, options, checksums)
        copyCommand = self.copyCommand.format_map(copyCommandDict)
        result = "#!/bin/bash\n" + copyCommand

        if _CheckExitCodeOption:
            result += """
            EXIT_STATUS=$?
            echo "gfal-copy exit status: $EXIT_STATUS"
            if [[ $EXIT_STATUS != 0 ]]; then
                echo "ERROR: gfal-copy exited with $EXIT_STATUS"
                echo "Cleaning up failed file:"
                {remove_command}
            fi
            exit $EXIT_STATUS
            """.format(remove_command=self.createRemoveFileCommand(targetPFN))

        return result

    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        Debug a failed gfal-cp command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for gfal-cp
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """

        copyCommandDict = self.buildCopyCommandDict(sourcePFN, targetPFN, options, checksums)
        copyCommand = self.copyCommand.format_map(copyCommandDict)

        result = "#!/bin/bash\n"
        result += """
        echo
        echo
        echo "-----------------------------------------------------------"
        echo "==========================================================="
        echo
        echo "Debugging information on failing gfal-copy command"
        echo
        echo "Current date and time: $(date +"%Y-%m-%d %H:%M:%S")"
        echo "gfal-copy command which failed: {copy_command}"
        echo "Hostname:   $(hostname -f)"
        echo "OS:  $(uname -r -s)"
        echo
        echo "GFAL environment variables:"
        env | grep ^GFAL
        echo
        echo "PYTHON environment variables:"
        env | grep ^PYTHON
        echo
        echo "LD_* environment variables:"
        env | grep ^LD_
        echo
        echo "gfal-copy location: $(which gfal-copy)"
        echo "Source PFN: {source}"
        echo "Target PFN: {destination}"
        echo
        echo
        echo "Information for credentials in the environment"
        echo "Bearer token content: $BEARER_TOKEN"
        echo "Bearer token file: $BEARER_TOKEN_FILE"
        echo "httokendecode path: $(which httokendecode)"
        echo "httokendecode: $httokendecode"
        echo
        echo "VOMS proxy info:"
        voms-proxy-info -all
        echo "==========================================================="
        echo "-----------------------------------------------------------"
        echo
        """.format(copy_command=copyCommand, source=copyCommandDict['source'], destination=copyCommandDict['destination'])
        return result

    def removeFile(self, pfnToRemove):
        """
        _removeFile_
        CleanUp pfn provided
        """
        if os.path.isfile(pfnToRemove):
            command = "/bin/rm -f {}".format(os.path.abspath(pfnToRemove))
        else:
            command = self.removeCommand.format(self.createFinalPFN(pfnToRemove))
        self.executeCommand(command)


registerStageOutImpl("gfal2", GFAL2Impl)



#!/usr/bin/env python
"""
_LoadFromID_

MySQL implementation of Fileset.LoadFromID
"""

__all__ = []



from WMCore.Database.DBFormatter import DBFormatter

class LoadFromID(DBFormatter):
    sql = """SELECT id, name, open, last_update FROM wmbs_fileset
             WHERE id = :fileset"""

    def formatDict(self, result):
        """
        _formatDict_

        Cast the id and last_update attributes to integers because the
        DBFormatter's formatDict() method changes all attributes to be
        strings.
        """
        formattedResult = DBFormatter.formatDict(self, result)[0]
        formattedResult["id"] = int(formattedResult["id"])
        formattedResult["last_update"] = int(formattedResult["last_update"])

        if int(formattedResult["open"]) == 1:
            formattedResult["open"] = True
        else:
            formattedResult["open"] = False

        return formattedResult

    def execute(self, fileset = None, conn = None, transaction = False):
        result = self.dbi.processData(self.sql, {"fileset": fileset},
                         conn = conn, transaction = transaction)
        return self.formatDict(result)



#!/usr/bin/env python

"""
WorkQueuManager test
"""

from builtins import range
from builtins import object
import threading
import unittest
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit
from WMQuality.Emulators.AnalyticsDataCollector.DataCollectorAPI import REQUEST_NAME_PREFIX, NUM_REQUESTS

from WMComponent.AnalyticsDataCollector.AnalyticsPoller import AnalyticsPoller
from WMComponent.AnalyticsDataCollector.DataCollectorEmulatorSwitch import EmulatorHelper

class MockLocalQService(object):

    def __init__(self):
        pass

    def getAnalyticsData(self):
        """
        This getInject status and input dataset from workqueue
        """
        status = {}
        inputDataset = {}
        for i in range(NUM_REQUESTS + 1):
            status['%s%s' % (REQUEST_NAME_PREFIX, i + 1)] = {'inQueue': 1, 'inWMBS': 1}
            inputDataset['%s%s' % (REQUEST_NAME_PREFIX, i + 1)] = 'inputdataset-%s' % (i + 1)

        return {'status': status, 'input_dataset': inputDataset}

class AnalyticsDataCollector_t(unittest.TestCase):
    """
    TestCase for WorkQueueManagerTest module
    """

    def setUp(self):
        """
        setup for test.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        self.testInit.setSchema(customModules=["WMComponent.DBS3Buffer"],
                                useDefault=False)
        self.reqmonDBName = "wmstat_t"
        self.localDBName = "wmstat_t_local"
        self.testInit.setupCouch(self.reqmonDBName, "WMStats")
        self.testInit.setupCouch(self.localDBName, "WMStats")
        self.testDir = self.testInit.generateWorkDir()
        EmulatorHelper.setEmulators(localCouch=True, reqMon=False, wmagentDB=True)
        return

    def tearDown(self):
        """
        Database deletion
        """
        threading.currentThread()

        self.testInit.delWorkDir()
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        EmulatorHelper.resetEmulators()
        return

    def getConfig(self):
        """
        _createConfig_

        General config file
        """
        couchURL = self.testInit.couchUrl
        config = self.testInit.getConfiguration()
        self.testInit.generateWorkDir(config)
        config.section_("Agent")
        config.Agent.hostName = "localhost"
        config.Agent.contact = "test@test.com"
        config.Agent.teamName = "testTeam"
        config.Agent.agentName = "testAgentName"
        config.Agent.useMsgService = False
        config.Agent.useTrigger = False
        config.Agent.useHeartbeat = False

        config.section_("General")
        config.General.centralWMStatsURL = "%s/%s" % (couchURL, self.reqmonDBName)

        config.section_("ACDC")
        config.ACDC.couchurl = couchURL
        config.ACDC.database = "acdc"

        config.component_("AnalyticsDataCollector")
        config.AnalyticsDataCollector.namespace = "WMComponent.AnalyticsDataCollector.AnalyticsDataCollector"
        config.AnalyticsDataCollector.componentDir = config.General.workDir + "/AnalyticsDataCollector"
        config.AnalyticsDataCollector.logLevel = "DEBUG"
        config.AnalyticsDataCollector.pollInterval = 240
        config.AnalyticsDataCollector.localCouchURL = "%s/%s" % (couchURL, "jobDump")
        config.AnalyticsDataCollector.localQueueURL = "%s/%s" % (couchURL, "workqueue")
        config.AnalyticsDataCollector.localWMStatsURL = "%s/%s" % (couchURL, self.localDBName)
        config.AnalyticsDataCollector.centralRequestDBURL = "%s/%s" % (couchURL, "requset_db_t")
        config.AnalyticsDataCollector.reqMonURL = "%s/%s" % (couchURL, self.reqmonDBName)
        config.AnalyticsDataCollector.RequestCouchApp = "ReqMgr"
        config.AnalyticsDataCollector.summaryLevel = "task"

        return config

    def testAnalyticsPoller(self):
        """
        Tests the components, as in sees if they load.
        Otherwise does nothing.
        """
        threading.currentThread()
        config = self.getConfig()
        analytics = AnalyticsPoller(config)

        analytics.setup(None)
        analytics.localQueue = MockLocalQService()
        analytics.algorithm(None)
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_DBSReader_t_

Unit test for the DBSConcurrency module
"""

import os
import json
import time
import logging
import unittest

# WMCore modules
from WMCore.Services.DBS.DBSConcurrency import getBlockInfo4PU
from WMCore.Services.pycurl_manager import getdata as multi_getdata
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase


class DBSConcurrencyTest(EmulatedUnitTestCase):
    """
    DBSConcurrencyTest class defines unit tests for DBS concurrent codebase
    """

    def setUp(self):
        """
        Initialization function
        """

        self.dbs = 'https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader'
        self.ckey = os.getenv('X509_USER_KEY')
        self.cert = os.getenv('X509_USER_CERT')
        logging.basicConfig()
        self.logger = logging.getLogger()
        self.logger.setLevel(logging.DEBUG)

    def testGetBlockInfoList(self):
        """
        Unit test for getBlockInfo4PU function
        """
        time0 = time.time()
        dataset = '/ZMM_13TeV_TuneCP5-pythia8/RunIIAutumn18DR-SNB_102X_upgrade2018_realistic_v17-v2/AODSIM'
        url = f"{self.dbs}/blocks?dataset={dataset}"
        self.logger.info(url)
        results = multi_getdata([url], self.ckey, self.cert)
        blocks = []
        for row in results:
            data = json.loads(row['data'])
            blocks = [r['block_name'] for r in data]
        elapsedTime = time.time() - time0
        self.logger.debug("for %s get %d in %s seconds", dataset, len(blocks), elapsedTime)
        self.assertTrue(len(blocks), 2)
        # call to DBS should be resovled within 1 second
        self.assertTrue(elapsedTime < 3)

        time0 = time.time()
        blockInfoList = getBlockInfo4PU(blocks, self.dbs, self.ckey, self.cert)
        for blk, row in blockInfoList.items():
            self.logger.debug("block %s, nfiles=%d, nevents=%d", blk, len(row['FileList']), row['NumberOfEvents'])
        elapsedTime = time.time() - time0
        self.logger.debug("Elapsed time: %d seconds", elapsedTime)
        # NOTE: if every DBS call spent 1 second, avg time for 10 calls will be around 1 second
        # therefore, we will test quite low number here
        self.assertTrue(elapsedTime < 3)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
sphinx-autopackage-script

This script parses a directory tree looking for python modules and packages and
creates ReST files appropriately to create code documentation with Sphinx.
It also creates a modules index (named modules.<suffix>).
"""
from __future__ import print_function

import argparse
import os

# Copyright 2008 Société des arts technologiques (SAT), http://www.sat.qc.ca/
# Copyright 2010 Thomas Waldmann <tw AT waldmann-edv DOT de>
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# automodule options
OPTIONS = ['members']
INIT = '__init__.py'


def makename(package, module):
    """Join package and module with a dot."""
    # Both package and module can be None/empty.
    if package:
        name = package
        if module:
            name += '.' + module
    else:
        name = module
    return name


def write_file(name, text, opts):
    """Write the output file for module/package <name>."""
    if opts.dryrun:
        return
    fname = os.path.join(opts.destdir, "%s.%s" % (name, opts.suffix))
    if not opts.force and os.path.isfile(fname):
        print('File %s already exists, skipping.' % fname)
    else:
        print('Creating file %s.' % fname)
        f = open(fname, 'w')
        f.write(text)
        f.close()


def format_heading(level, text):
    """Create a heading of <level> [1, 2 or 3 supported]."""
    underlining = ['=', '-', '~', ][level - 1] * len(text)
    return '%s\n%s\n\n' % (text, underlining)


def format_directive(module, package=None):
    """Create the automodule directive and add the options."""
    directive = '.. automodule:: %s\n' % makename(package, module)
    for option in OPTIONS:
        directive += '    :%s:\n' % option
    return directive


def create_module_file(package, module, opts):
    """Build the text of the file and write the file."""
    text = format_heading(1, ':mod:`%s` Module' % module)
    text += format_directive(module, package)
    filename = makename(package, module)
    write_file(filename, text, opts)
    return filename


def create_submodule_file(master_package, subroot, package, py_file, opts):
    is_package = py_file == INIT
    py_file = os.path.splitext(py_file)[0]
    py_path = makename(subroot, py_file)
    if is_package:
        text = format_heading(2, ':mod:`%s` Package' % package)
        text += format_directive(is_package and subroot or py_path, master_package)
        text += '\n'
        return (text, '')
    else:
        return ('', create_module_file(subroot, py_file, opts))


def create_package_file(root, master_package, subroot, py_files, opts, subs):
    """Build the text of the file and write the file."""
    package = os.path.split(root)[-1]
    text = format_heading(1, '%s Package' % package)
    toc = ''

    # add each package's module
    for py_file in py_files:
        if shall_skip(os.path.join(root, py_file)):
            continue
        headpart, tocpart = create_submodule_file(master_package, subroot, package, py_file, opts)
        if headpart:
            text += headpart
        if tocpart:
            if not toc:
                toc = format_heading(2, ':mod:`%s` Modules' % package)
                toc += '.. toctree::\n\n'
            toc += '    %s\n' % tocpart

    # build a list of directories that are packages (they contain an INIT file)
    subs = [sub for sub in subs if os.path.isfile(os.path.join(root, sub, INIT))]
    # if there are some package directories, add a TOC for theses subpackages
    if subs:
        toc += '\n'
        toc += format_heading(2, 'Subpackages')
        toc += '.. toctree::\n\n'
        for sub in subs:
            toc += '    %s.%s\n' % (makename(master_package, subroot), sub)
        toc += '\n'

    write_file(makename(master_package, subroot), text + toc, opts)


def create_modules_toc_file(master_package, modules, opts, name='modules'):
    """
    Create the module's index.
    """
    text = format_heading(1, '%s Modules' % opts.header)
    text += '.. toctree::\n'
    text += '   :maxdepth: %s\n\n' % opts.maxdepth

    modules.sort()
    prev_module = ''
    for module in modules:
        # look if the module is a subpackage and, if yes, ignore it
        if module.startswith(prev_module + '.'):
            continue
        prev_module = module
        text += '   %s\n' % module

    write_file(name, text, opts)


def shall_skip(module):
    """
    Check if we want to skip this module.
    """
    # skip it, if there is nothing (or just \n or \r\n) in the file
    return os.path.getsize(module) < 3


def recurse_tree(path, excludes, opts):
    """
    Look for every file in the directory tree and create the corresponding
    ReST files.
    """
    # use absolute path for root, as relative paths like '../../foo' cause
    # 'if "/." in root ...' to filter out *all* modules otherwise
    path = os.path.abspath(path)
    # check if the base directory is a package and get is name
    if INIT in os.listdir(path):
        package_name = path.split(os.path.sep)[-1]
    else:
        package_name = None

    toc = []
    tree = os.walk(path, False)
    for root, subs, files in tree:
        # keep only the Python script files
        py_files = sorted([f for f in files if os.path.splitext(f)[1] == '.py'])
        if INIT in py_files:
            py_files.remove(INIT)
            py_files.insert(0, INIT)
        # remove hidden ('.') and private ('_') directories
        subs = sorted([sub for sub in subs if sub[0] not in ['.', '_']])
        # check if there are valid files to process
        # TODO: could add check for windows hidden files
        if "/." in root or "/_" in root \
                or not py_files \
                or is_excluded(root, excludes):
            continue
        if INIT in py_files:
            # we are in package ...
            if (  # ... with subpackage(s)
                            subs
                        or
                        # ... with some module(s)
                                len(py_files) > 1
                    or
                    # ... with a not-to-be-skipped INIT file
                        not shall_skip(os.path.join(root, INIT))
            ):
                subroot = root[len(path):].lstrip(os.path.sep).replace(os.path.sep, '.')
                create_package_file(root, package_name, subroot, py_files, opts, subs)
                toc.append(makename(package_name, subroot))
        elif root == path:
            # if we are at the root level, we don't require it to be a package
            for py_file in py_files:
                if not shall_skip(os.path.join(path, py_file)):
                    module = os.path.splitext(py_file)[0]
                    create_module_file(package_name, module, opts)
                    toc.append(makename(package_name, module))

    # create the module's index
    if not opts.notoc:
        create_modules_toc_file(package_name, toc, opts)


def normalize_excludes(rootpath, excludes):
    """
    Normalize the excluded directory list:
    * must be either an absolute path or start with rootpath,
    * otherwise it is joined with rootpath
    * with trailing slash
    """
    sep = os.path.sep
    f_excludes = []
    for exclude in excludes:
        if not os.path.isabs(exclude) and not exclude.startswith(rootpath):
            exclude = os.path.join(rootpath, exclude)
        if not exclude.endswith(sep):
            exclude += sep
        f_excludes.append(exclude)
    return f_excludes


def is_excluded(root, excludes):
    """
    Check if the directory is in the exclude list.

    Note: by having trailing slashes, we avoid common prefix issues, like
          e.g. an exlude "foo" also accidentally excluding "foobar".
    """
    sep = os.path.sep
    if not root.endswith(sep):
        root += sep
    for exclude in excludes:
        if root.startswith(exclude):
            return True
    return False


def main():
    """
    Parse and check the command line arguments.
    """
    parser = argparse.ArgumentParser(usage="""usage: %prog [options] <package path> [exclude paths, ...]

Note: By default this script will not overwrite already created files.""")
    parser.add_argument("-n", "--doc-header", action="store", dest="header",
                        help="Documentation Header (default=Project)", default="Project")
    parser.add_argument("-d", "--dest-dir", action="store", dest="destdir", help="Output destination directory",
                        default="")
    parser.add_argument("-s", "--suffix", action="store", dest="suffix", help="module suffix (default=txt)",
                        default="txt")
    parser.add_argument("-m", "--maxdepth", action="store", dest="maxdepth",
                        help="Maximum depth of submodules to show in the TOC (default=4)", type=int, default=4)
    parser.add_argument("-r", "--dry-run", action="store_true", dest="dryrun",
                        help="Run the script without creating the files")
    parser.add_argument("-f", "--force", action="store_true", dest="force", help="Overwrite all the files")
    parser.add_argument("-t", "--no-toc", action="store_true", dest="notoc",
                        help="Don't create the table of content file")
    (opts, args) = parser.parse_known_args()
    if not args:
        parser.error("package path is required.")
    else:
        rootpath, excludes = args[0], args[1:]
        if os.path.isdir(rootpath):
            # check if the output destination is a valid directory
            if opts.destdir and os.path.isdir(opts.destdir):
                excludes = normalize_excludes(rootpath, excludes)
                recurse_tree(rootpath, excludes, opts)
            else:
                print('%s is not a valid output destination directory.' % opts.destdir)
        else:
            print('%s is not a valid directory.' % rootpath)


if __name__ == '__main__':
    main()
